from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, VariantRule, Cfg0ConfiguratorPerspective, WorkspaceObject
from typing import Dict, List
from tcsoa.gen.Internal.ProductConfiguratorAw._2018_05.ConfiguratorManagement import SelectionsSummary, VariantExpressionFilters, Violation, ViewModelObjectLabelType
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ApplicationConfigExpression(TcBaseObj):
    """
    Structure representing a single variant expression.
    
    :var formula: The persistent configuration formula string.
    :var exprID: The application expression identifier used to distinguish the current expression from other variant
    expressions authored for the same business object.
    :var exprType: Expression type represents expression category and the intent of this expression. The valid values
    are as follows: 9  - Variant Condition, 18 - Variant Rule, 25 &ndash; Applicability,44 &ndash; Subject , 28 -
    Inclusive Constraint,29 - Exclusive Constraint,37 - Default Rule,41 - Availability Rule etc. which are defined in
    cfg_tokens.h file.
    :var configExprSets: A list of 'ConfigExpressionSet' which are relevant to the business object. The variant
    expression sets are combined by OR operator.
    """
    formula: str = ''
    exprID: str = ''
    exprType: int = 0
    configExprSets: List[ConfigExpressionSet] = ()


@dataclass
class ConfigExpressionSection(TcBaseObj):
    """
    Structure defines a Boolean expression to be used for configuration purposes.
    
    :var exprType: Expression type represents expression category and the intent of this expression. The valid values
    are as follows: 9 - Variant Condition, 18 - Variant Rule, 25 &ndash; Applicability, 44 &ndash; Subject, 28 -
    Inclusive Constraint, 29 - Exclusive Constraint, 37 - Default Rule, 41 - Availability Rule. These constants are
    defined in cfg_tokens.h file.
    :var formula: Configuration formula for a 'ConfigExpressionSection'.
    :var subExpressions: A list of sub-expressions in a Boolean variant expression. Either formula or sub expressions
    are populated in a 'ConfigExpressionSection' at a time.
    """
    exprType: int = 0
    formula: str = ''
    subExpressions: List[ConfigSubExpression] = ()


@dataclass
class SetVariantExpressionDataInput(TcBaseObj):
    """
    Input containing the user selections that represent variability to be set on variant configurable objects for e.g
    VariantRule.
    
    :var selectedExpressions: A map (string, list of ApplicationConfigExpressions ) of affected object UID to the list
    of its selected variant expressions. Affected object is any object for which selections are stored. Valid values
    for affected object are UIDs of VariantRule, Awb0Element or any configurable object.
    :var configPerspective: Cfg0ConfiguratorPerspective object is used as communication mechanism for the configuration
    and the valid configurator contexts across the Product Configurator and its consuming applications such as 4G
    Designer, Structure Manager. If it is empty then the config perspective is obtained from the selected objects
    :var requestInfo: Map (string, list of strings ) of request names and value pairs. Allowed names are: 
    -"requestType" : This gives the operation to be handled for the SOA call. 
    [ "requestType" -> { ("viewPackage") | ("selectPackage") | ("addPackage") | ("cancelPackage") | ("getConfig") |
    ("validate") | ("switchToManual") | ("expand") }]
    - "selectedPackage" : nodeID of selected package feature object. This is required in case of requestType is one of
    "viewPackage", "selectPackage", "cancelPackage", "addPackage".
    - "configurationControlMode" is the configuration view mode in manual mode invalid selections are allowed.
    ["configurationControlMode" -> { "guided" | "manual" }]
    - "severity" : This will give information what all violations you want to show.
    ["severity" &#61672; { "error" | "warn" | "info" }]
    -"viewName" : This will hold the current configuration view name.
    [" viewName" &#61672; { "guidedConfig" | "manualWithError" | "manualWithExpand" | "package"}]
    """
    selectedExpressions: SelectedExpressions = None
    configPerspective: Cfg0ConfiguratorPerspective = None
    requestInfo: StringMap3 = None


@dataclass
class UserSelection(TcBaseObj):
    """
    Structure representing a single user selection contributing a violation. Other information will be populated in
    'ViewModelObjectMap2'.
    
    :var groupUid: UID of the group corresponding to feature contributing to violation.
    :var configExpressionTerm: Holds a single selection. The nodeUid in the 'configExpressionTerm' is the unique ID to
    represent a node in the hierarchy which can be family (Cfg0AbsFamily) or a feature (Cfg0AbsFeature).
    """
    groupUid: str = ''
    configExpressionTerm: ConfigExpressionTerm = None


@dataclass
class ValidateProductConfigsInput(TcBaseObj):
    """
    The input containing the configurator perspective and the SelectedExpressions to validate by applying the
    constraints for the input perspective.
    
    :var configPerspective: Instance of Cfg0ConfiguratorPerspective business object.
    :var applyConstraints: The constraint rules application. The parameter is a bitwise &lsquo;OR&rsquo; of the
    following values: 
    0: Disable configurator constraint evaluation. 
    1: Enable configurator constraint evaluation.
    2: Skip constraints if they only reference unset families.
    4: Skip constraints that don't reference all Expression families.
    8: Report warnings in addition to errors.
    16: Report informational messages.
    64: Treat constraints with warning severity as if they had error severity.
    256: Skip allocation constraints in solve.
    1024: Skip criteria update based on validation rules (For example configurator exclusion rule). If this flag is not
    set then the operation will update the variant criteria after applying validation rules.
    2048: Enable Availability Constraint evaluation. 
    4096: Include Default Rules in the violations.
    :var requestInfo: Map (string, list of strings) of request names and value pairs. 
    
    1.  "profileSetting"  contains the information regarding the setting of different severity information (Error,
    warning, info) in JSON format which is taken into consideration for for the validation.
    :var selectedExpressions: A map (string, list of ApplicationConfigExpressions) containing the expressions in
    PCAGrid format, which needs to be validated.
    """
    configPerspective: Cfg0ConfiguratorPerspective = None
    applyConstraints: int = 0
    requestInfo: StringMap3 = None
    selectedExpressions: SelectedExpressions = None


@dataclass
class ConfigExpressionSet(TcBaseObj):
    """
    Structure containing the collection of variant expressions referred as set for the business
    object. A set collects different type of variant expressions such as applicability and subject. These will be
    joined together by AND operation and evaluated as part of single expression.
    
    :var configExprSections: A list of 'ConfigExpressionSection' structures which are collected as a set for the
    business object. Each structure defines a variant expression which can be used for the configuration purposes.
    """
    configExprSections: List[ConfigExpressionSection] = ()


@dataclass
class ValidateProductConfigsResponse2(TcBaseObj):
    """
    It is the structure containing response of operation validateProductConfiuration2.
    
    :var validateProductConfigurationOutputs: A map (string, ValidateProductConfigurationOutput2) containing the
    Selected object UID to its ValidateProductConfigurationOutput2 response structure.
    :var responseInfo: A map (string, list of strings) containing the additional informations which server wants to
    send about the operation. 
    Following information is sent by the server
    incompleteFamilies &ndash; UID of the families for which no selection is made by the user and system could not
    select any valid value by considering the constraints for the input expression. 
    completenessStatus &ndash; Provide additional information about the completeness of the input expression. Following
    are the valid values.
    ValidAndIncomplete &ndash; Indicates that the current input expression is valid but not every family have been
    selected.
    ValidAndIncomplete &ndash; Indicates that the current input expression is valid and have the selection for every
    family.
    Invalid &ndash; Indicates that the input expression is invalid.
    :var serviceData: ServiceData containing partial exceptions, if any.
    """
    validateProductConfigurationOutputs: AffectedObjectToValidateProductConfiguration = None
    responseInfo: StringMap3 = None
    serviceData: ServiceData = None


@dataclass
class ValidateProductConfigurationOutput2(TcBaseObj):
    """
    It is the structure containing information about the expanded expression and criteria status for the corresponding
    configurable object.
    
    :var criteriaStatus: Indicate if input configuration is valid or not. If true, input configuration is valid;
    otherwise, input configuration is invalid.
    :var valueToViolations: A map (string, Violations) of all violated values. The key is the UID of the feature
    (Cfg0AbsFeature) for the violations are reported by the system.
    :var expression: Expression in PCAGrid format which contains the expanded selections that is in accordance with the
    given input selections and the rules in the context.
    """
    criteriaStatus: bool = False
    valueToViolations: ValueToViolations2 = None
    expression: ApplicationConfigExpression = None


@dataclass
class VariabilityTreeData(TcBaseObj):
    """
    Tree structure representing variant data. This holds list of a structure VariabiltyNode. Each element will
    represent UID of the group, family and feature in the response and list of UIDs of all children. For features, no
    children data is populated.
    This structure will be populated for current scope or for all scopes based on view\mode information received in the
    requestInfo.
    
    In Tree Mode &ndash; Single SVR,  parent and child information of all scopes to be populated.
    In Tree Mode &ndash; Multiple SVR, parent and child information of all scopes to be populated.
    In existing Mode &ndash; Single SVR, parent and child information of current scope to be populated.And only list of
    all other parents is sufficient.
    
    :var variabiltyNodes: Represents UID of and list of UIDs of all children along with other properties to know
    additional information.
    """
    variabiltyNodes: List[VariabiltyNode] = ()


@dataclass
class VariabiltyNode(TcBaseObj):
    """
    Structure representing UID of Object and list of UIDs of all children along with other properties to know
    additional information.
    
    :var nodeUid: The UID of the object to be rendered.
    :var isExpanded: If true, the objects are expanded.
    :var childrenUids: The list of UIDs of children. This can contain value texts in case of free form features.
    :var props: The node specific meta information. For example &ndash; 1. allowed selection states in case of guided
    mode for family as well as feature. 2. For free form family nodeUid will represent information like familyUid and
    prop contains isFreeForm=true, children contains values texts like 10,20.
    """
    nodeUid: str = ''
    isExpanded: bool = False
    childrenUids: List[str] = ()
    props: StringMap3 = None


@dataclass
class VariantConfigurationViewIn2(TcBaseObj):
    """
    The input containing the selected context and selections to fetch the available variability.
    
    :var configPerspective: The configurator perspective. When this operation is invoked first time, this parameter
    would be null. But in all subsequent calls, this parameter must be populated with the configurator perspective
    retrieved in the response.
    :var selectedContext: The selected object for which variant configurator context to be retrieved. It can be a
    runtime business object (instance of Awb0Element) or persistent variant configurable business object (instance of
    Mdl0ConditionalElement). This parameter is ignored if configPerspective is passed in input.
    :var activeVariantRules: An optional list of active VariantRule whose expression can be used as input selections
    for computing next valid selections. The parameter is ignored if the &lsquo;selections&rsquo; is populated.
    :var scopes: A list of UIDs of Scope (Cfg0AbsFamilyGroup, Ptn0Partition) for which the next valid selections must
    to be retrieved. If this parameter is empty, then option values from model group (Cfg0AbsFamilyGroup) are retrieved.
    :var selectedExpressions: A map (string, list of 'ApplicationConfigExpressions' ) of affected object UID to the
    list of its selected variant expressions. Affected object is any object for which selections are stored. Valid
    values for affected object are UIDs of VariantRule, Awb0Element or any configurable object.
    :var payloadStrings: A map (string, list of strings) represents the payload that is transferred between client and
    server for the current session. The operation would use the payload as the state information to process the request.
    The client should pass the same payloadStrings that is returned by previous call if the state information is to be
    maintained for the current request.
    Valid map:
    "violation" -> { "list of nodeIDs of viewModelObject which hold the violation" }
    :var requestInfo: Map (string, string) of request names and value pairs. Allowed names are: 
    -"requestType" : This gives the operation to be handled for the SOA call. 
    [ "requestType" -> { ("viewPackage") | ("selectPackage") | ("addPackage") | ("cancelPackage") | ("getConfig") |
    ("validate") | ("switchToManual") | ("expand") }]
    - "selectedPackage" : nodeID of selected package feature object. This is required in case of requestType is one of
    "viewPackage", "selectPackage", "cancelPackage", "addPackage".
    - "configurationControlMode" is the configuration view mode in manual mode invalid selections are allowed.
    ["configurationControlMode" -> { "guided" | "manual" }]
    - "severity" : This will give information what all violations you want to show.
    ["severity" &#61672; { "error" | "warn" | "info" }]
    -"viewName" : This will hold the current configuration view name.
    [" viewName" &#61672; { "guidedConfig" | "manualWithError" | "manualWithExpand" | "package"}]
    """
    configPerspective: Cfg0ConfiguratorPerspective = None
    selectedContext: BusinessObject = None
    activeVariantRules: List[VariantRule] = ()
    scopes: List[str] = ()
    selectedExpressions: SelectedExpressions = None
    payloadStrings: StringMap3 = None
    requestInfo: StringMap3 = None


@dataclass
class ConfigExpressionTerm(TcBaseObj):
    """
    Structure representing a single selection which will hold variant option and its selection state.
    
    :var nodeUid: The unique ID to represent a node in the hierarchy which can be family (Cfg0AbsFamily) or a feature
    (Cfg0AbsFeature).
    :var valueText: The string value of feature to be used in case of free form families.
    :var family: The UID of the family (Cfg0AbsFamily).
    :var familyId: The unique Object identifier for the Family (Cfg0AbsFamily).
    :var familyNamespace: The family namespace by which the family object is uniquely identified. This is used for
    unconfigured families where no UID is available.
    :var selectionState: The state of object selection. Selection state values are:
    0 - No Selection.
    1 - User positive Selection.
    2 - User negative Selection.
    5 - Default positive Selection by System.
    6 - Default negative Selection by system.
    9 - positive Selection by System. 
    10 - negative Selection by System.
    :var props: A map (string, list of strings) to store meta data about config object. It is used to indicate free
    form, unconfigured selection.
    """
    nodeUid: str = ''
    valueText: str = ''
    family: str = ''
    familyId: str = ''
    familyNamespace: str = ''
    selectionState: int = 0
    props: StringMap3 = None


@dataclass
class VariantConfigurationViewResponse2(TcBaseObj):
    """
    The output containing the available variability for the input list of variant option value selections.
    
    :var configPerspective: The configurator perspective containing all information about the current Configurator
    Context, revision rule and effectivity. All further communications with the server to retrieve variant
    configuration data must use this object.
    :var scopes: UID of the current expanded group or partition.
    :var serviceData: serviceData
    :var variabilityTreeData: Tree structure representing variant data.
    :var viewModelObjectMap: A map (string, 'ViewModelObject2') of object UID to its 'ViewModelObject2'.
    :var selectionsSummary: The summary of all selections in displayable format.
    :var selectedExpressions: A map (string, list of 'ApplicationConfigExpressions') of affected object UID to the list
    of its selected variant expressions. Affected object is any object for which selections are stored. Valid values
    for affected object are UIDs of VariantRule, Awb0Element or any configurable object.
    :var labels: 'ViewModelObjectLabelMap' of labels.
    :var violationCombo: Structure representing a single problem\violation and its contributing user selections as well
    contributing rules.
    :var payloadStrings: A map (string, list of strings) represents the payload that is transferred between client and
    server for the current session. The operation would use the payload as the state information to process the request.
    :var responseInfo: Map (string, string) of response names and value pairs. Allowed names are:
    -"requestType" : This gives the operation to be handled for the SOA call. 
    [ "requestType" -> { ("viewPackage") | ("selectPackage") | ("addPackage") | ("cancelPackage") | ("getConfig") |
    ("validate") | ("switchToManual") | ("expand") }]
    - "selectedPackage" : nodeID of selected package feature object. This is required in case of requestType is one of
    "viewPackage", "selectPackage", "cancelPackage", "addPackage".
    - "configurationControlMode" is the configuration view mode in manual mode invalid selections are allowed.
    ["configurationControlMode" -> { "guided" | "manual" }]
    - "severity" : This will give information what all violations you want to show.
    ["severity" &#61672; { "error" | "warn" | "info" }]
    """
    configPerspective: Cfg0ConfiguratorPerspective = None
    scopes: List[str] = ()
    serviceData: ServiceData = None
    variabilityTreeData: VariabilityTreeData = None
    viewModelObjectMap: ViewModelObjectMap2 = None
    selectionsSummary: SelectionsSummary = None
    selectedExpressions: SelectedExpressions = None
    labels: ViewModelObjectLabelMap2 = None
    violationCombo: List[ViolationCombo] = ()
    payloadStrings: StringMap3 = None
    responseInfo: StringMap3 = None


@dataclass
class VariantExpressionDataInput2(TcBaseObj):
    """
    The input structure for the getVariantExpressionData operation containing either the selected objects or current
    expanded families and optionally containing configurator context, configurator perspective, and variant expression
    filters.
    
    :var configContextProvider: The configurator context provider object. It is optional. If not null, it is used to
    find configurator context object. Valid input types are ItemRevision, Awb0Element and Mdl0ConditionalElement. This
    parameter is ignored if configContext or configPerspective parameters are passed.
    :var configContext: The variant configurator context. Valid input types are Item or ItemRevision. This parameter is
    ignored if the configPerspective parameter is passed.
    :var configPerspective: Input Cfg0ConfiguratorPerspective. It is optional. Cfg0ConfiguratorPerspective object is
    used as communication mechanism for the configuration and the valid configurator contexts across the Product
    Configurator and its consuming applications such as 4G Designer, Structure Manager. If it is empty then the config
    perspective is obtained from the selected objects.
    :var selectedObjects: The list of selected business objects. If this parameter is empty then
    currentExpandedFamilies and configPerspective should be provided. The supported business object types are
    Awb0Element, Mdl0ConditionalElement and variant configurable business objects - BOMLine, VariantRule.
    :var currentExpandedFamilies: The list of expanded option families. If the input contains only
    currentExpandedFamilies and configPerspective as non-empty input then operation will return only the Option values
    of the provided families.
    :var filters: The input filters that govern the option family and value response. For instance input filter may
    contain option filter (&lsquo;showCurrentElements&rsquo;, &lsquo;showFamilies&rsquo; ) or intent filters (from
    Cfg0ObjectIntentions List Of Values).
    :var requestInfo: Map (string, list of strings ) of request names and value pairs. Allowed names are: 
    -"requestType" : This gives the operation to be handled for the SOA call. 
    [ "requestType" -> { ("viewPackage") | ("selectPackage") | ("addPackage") | ("cancelPackage") | ("getConfig") |
    ("validate") | ("switchToManual") | ("expand") }]
    - "selectedPackage" : nodeID of selected package feature object. This is required in case of requestType is one of
    "viewPackage", "selectPackage", "cancelPackage", "addPackage".
    - "configurationControlMode" is the configuration view mode in manual mode invalid selections are allowed.
    ["configurationControlMode" -> { "guided" | "manual" }]
    - "severity" : This will give information what all violations you want to show.
    ["severity" &#61672; { "error" | "warn" | "info" }]
    -"viewName" : This will hold the current configuration view name.
    [" viewName" &#61672; { "guidedConfig" | "manualWithError" | "manualWithExpand" | "package"}]
    """
    configContextProvider: BusinessObject = None
    configContext: WorkspaceObject = None
    configPerspective: Cfg0ConfiguratorPerspective = None
    selectedObjects: List[BusinessObject] = ()
    currentExpandedFamilies: List[str] = ()
    filters: VariantExpressionFilters = None
    requestInfo: StringMap3 = None


@dataclass
class VariantExpressionDataResponse2(TcBaseObj):
    """
    Variant expression response for the selected objects.
    
    :var variabilityTreeData: Tree structure representing variant data.
    :var viewModelObjectMap: A map (string, ViewModelObject2) of object UID to its ViewModelObject2 object.
    :var selectedExpressions: A map (string, list of ApplicationConfigExpressions) of affected object UID to the list
    of its selected variant expressions. Affected object is any object for which selections are stored. Valid values
    for affected object are UIDs of VariantRule, Awb0Element or any other object which have variant configurable
    behavior attached.
    :var configPerspective: Output configurator perspective. Configurator Perspective object is used as communication
    mechanism for the configuration and the valid configurator contexts across the Product Configurator and its
    consuming applications such as 4G Designer, Structure Manager.
    :var filters: The filters applied on the output to render the option families and values.
    :var responseInfo: Map (string, list of strings) of response names and value pairs. 
    1. "configurationControlMode" is the configuration control view.
    Supported values are: "guided" and "manual".
    guided: In this mode, configuration is always valid. This mode is used to configure products by optimizing the
    required input by pre-populating choices and at the same time provide maximum flexibility to users to get exactly
    what they want by overriding the system selections.
    manual: Manual mode does not give system selections unless user validates or applies his configuration.
    
    2."severity": This gives information what all violations to be returned.
    Supported values are: "error", "warn" and "info".
    
    3. "viewName": This holds the current configuration view name which decides how to display configuration data.
    Supported values are: "listView" and "treeView".
    listView: In this view Configurator data is displayed in simple list format.
    treeView: In this view Configurator data is displayed in tree format. Families are shown as children of Groups.
    Features are shown as children of respective families.
    :var serviceData: Service Data containing partial errors, if any.
    """
    variabilityTreeData: VariabilityTreeData = None
    viewModelObjectMap: ViewModelObjectMap2 = None
    selectedExpressions: SelectedExpressions = None
    configPerspective: Cfg0ConfiguratorPerspective = None
    filters: VariantExpressionFilters = None
    responseInfo: StringMap3 = None
    serviceData: ServiceData = None


@dataclass
class ViewModelObject2(TcBaseObj):
    """
    The generic object that is used to represent a single user facing object and its properties.
    It encapsulates a model object in server side.
    This object represents each object which is visible on UI like Scope, Feature, OptionFamily.
    
    :var sourceUid: The UID of the object to be rendered. Valid values are either valid UIDs of configurable object
    along with context UIDs.
    :var displayName: The display name of the object.
    :var sourceType: Type of the 'ViewModelObject'2. Valid values are following Business object types:
    Cfg0AbsFamilyGroup, Cfg0PackageOptionValue, Cfg0AbsValue, Cfg0AbsFamily.
    :var props: Map (string, list of strings) of 'ViewModelObject'2 property and value. Valid keys are: "isFreeForm",
    "isMultiSelect", "isPackage"
    """
    sourceUid: str = ''
    displayName: str = ''
    sourceType: str = ''
    props: StringMap3 = None


@dataclass
class ConfigSubExpression(TcBaseObj):
    """
    Structure containing map (string, list of 'ConfigExpressionTerms') of all option values selected by user per
    Cfg0AbsFamily.
    
    :var configExpressionGroupMap: Map (string, list of 'ConfigExpressionTerms') of all option values selected by user
    per family. 
    Here key is the UID of the family (Cfg0AbsFamily). For unconfigured and unassigned families, key is namespace and
    display name of the family.
    """
    configExpressionGroupMap: ConfigExpressionGroup = None


@dataclass
class ViolationCombo(TcBaseObj):
    """
    Structure representing a single problem\violation and its contributing user selections as well contributing rules.
    
    :var userSelections: A list of user selections contributing to violation.
    :var ruleUids: A list of UIDs of contributing rules.  More information is retrieved from 'ViewModelObject2'.
    :var props: A map (string, list of strings) to store meta information about selections or rules. For example, when
    the rule does not have access 'props' contains message as &lsquo;access denied&rsquo;.
    """
    userSelections: List[UserSelection] = ()
    ruleUids: List[str] = ()
    props: StringMap3 = None


"""
It is a map containing information about the result of validate product configuration for the corresponding affected object.
"""
AffectedObjectToValidateProductConfiguration = Dict[str, ValidateProductConfigurationOutput2]


"""
Map (string, list of 'ConfigExpressionTerms') of all option values selected by user per family. 
Here key is the UID of the family (Cfg0AbsFamily). For unconfigured and unassigned families, key is namespace and display name of the family.
"""
ConfigExpressionGroup = Dict[str, List[ConfigExpressionTerm]]


"""
A map (string, Violations) of all violated values.
"""
ValueToViolations2 = Dict[str, Violation]


"""
Map (string, list of ViewModelObjectLabelType) which will hold 'ViewModelObjects' for violations and package.

The valid keys are "violations", "package"
The value is ViewModelObjectLabelType which contains the 'ViewModelObjects' in a map where key names are nodeID and values are corresponding "violation" or "package"' ViewModelObjects'.
"""
ViewModelObjectLabelMap2 = Dict[str, List[ViewModelObjectLabelType]]


"""
Map (string, 'ViewModelObject'2) of all config Objects. 
The key represents the UID of the 'ViewModelObject'2.
This map is useful for the client to obtain 'ViewModelObject'2 properties like its type, display name and other properties.
"""
ViewModelObjectMap2 = Dict[str, ViewModelObject2]


"""
Map (string , list of variant expressions). Here, key is UID of the affected object for which selections are stored. Display name and other properties of affected object are stored in 'ViewModelObjectMap2'.
"""
SelectedExpressions = Dict[str, List[ApplicationConfigExpression]]


"""
Map (string, list of strings) to hold any property and its values.
"""
StringMap3 = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Core._2007_06.LOV import AttachedLOVsResponse, LOVInfo
from tcsoa.gen.Core._2007_06.PropDescriptor import PropDescInfo, AttachedPropDescsResponse
from tcsoa.gen.Core._2007_06.DataManagement import WhereReferencedByRelationNameResponse, GetAvailableTypesResponse, ExpandGRMRelationsResponse, ValidateItemIdsAndRevIdsResponse, ExpandGRMRelationsPref, DatasetTypeInfoResponse, ValidateIdsInfo, BaseClassInput, WhereReferencedByRelationNameInfo, PurgeSequencesInfo, SetOrRemoveImmunityInfo
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def purgeSequences(cls, objects: List[PurgeSequencesInfo]) -> ServiceData:
        """
        Given a list of ItemRevision sequences, this opertion is used ot perform per the following criteria: 
        - If the input object is the latest sequence of an ItemRevision, all previous sequences will be purged. 
        - If the input object is not the latest sequence of the ItemRevision and the 'validateLatestFlag' is false, the
        input object will be purged.
        - If the input object is not the latest sequence of the ItemRevision and the 'validateLatestFlag' is true, the
        'ServiceData' will be updated with an error.
        
        """
        return cls.execute_soa_method(
            method_name='purgeSequences',
            library='Core',
            service_date='2007_06',
            service_name='DataManagement',
            params={'objects': objects},
            response_cls=ServiceData,
        )

    @classmethod
    def setOrRemoveImmunity(cls, objects: List[SetOrRemoveImmunityInfo]) -> ServiceData:
        """
        This operation is used to add or remove immunity for each object in the input list according to the value of
        the associated 'setOrRemoveFlag'.  A value of true indicates to apply immunity to the object.  A value of false
        indicates that immunity should be removed from the object.
        """
        return cls.execute_soa_method(
            method_name='setOrRemoveImmunity',
            library='Core',
            service_date='2007_06',
            service_name='DataManagement',
            params={'objects': objects},
            response_cls=ServiceData,
        )

    @classmethod
    def validateItemIdsAndRevIds(cls, infos: List[ValidateIdsInfo]) -> ValidateItemIdsAndRevIdsResponse:
        """
        Validates the item ID and revision ID based on the naming rules and user exit callbacks.
        """
        return cls.execute_soa_method(
            method_name='validateItemIdsAndRevIds',
            library='Core',
            service_date='2007_06',
            service_name='DataManagement',
            params={'infos': infos},
            response_cls=ValidateItemIdsAndRevIdsResponse,
        )

    @classmethod
    def whereReferencedByRelationName(cls, inputs: List[WhereReferencedByRelationNameInfo], numLevels: int) -> WhereReferencedByRelationNameResponse:
        """
        Finds the objects that reference a given object by a specific relation. The input object will be the secondary
        object for that relation.  It does not return relations where the given input object is the primary object for
        the relation.  The Datamanagement service operation expandGRMRelationsForPrimary can be used to return the
        relations where the input object is the primary object and the objects which are the secondary object for the
        relation
        
        Use cases:
        Use case 1: Use this operation to find the objects referencing the input object by a specific relation.
        Use case 2: Use this operation to find objects of a specific type that reference the input object by a specific
        relation.
        Use case 3: Use this operation to find objects of a specific type referencing the input object.
        """
        return cls.execute_soa_method(
            method_name='whereReferencedByRelationName',
            library='Core',
            service_date='2007_06',
            service_name='DataManagement',
            params={'inputs': inputs, 'numLevels': numLevels},
            response_cls=WhereReferencedByRelationNameResponse,
        )

    @classmethod
    def expandGRMRelationsForPrimary(cls, primaryObjects: List[BusinessObject], pref: ExpandGRMRelationsPref) -> ExpandGRMRelationsResponse:
        """
        Returns the secondary objects related to the input object for the given list of properties / relations and
        other side object types.  If no properties/relations or other side objects types are input, then all related
        objects will be returned.  In addition, for performance, if an Item is the output of the expansion, then its
        associated Item Revisions and the Datasets for those Item Revisions will be returned.  Similarly, if an Item
        Revision is the output of the expansion, then its associated Datasets will be returned.
        """
        return cls.execute_soa_method(
            method_name='expandGRMRelationsForPrimary',
            library='Core',
            service_date='2007_06',
            service_name='DataManagement',
            params={'primaryObjects': primaryObjects, 'pref': pref},
            response_cls=ExpandGRMRelationsResponse,
        )

    @classmethod
    def expandGRMRelationsForSecondary(cls, secondaryObjects: List[BusinessObject], pref: ExpandGRMRelationsPref) -> ExpandGRMRelationsResponse:
        """
        Returns the primary objects related to the input object for the given list of properties / relations and other
        side object types.  If no properties/relations or other side objects types are input, then all related objects
        will be returned.  In addition, for performance, if an Item is the output of the expansion, then its associated
        Item Revisions and the Datasets for those Item Revisions will be returned.  Similarly, if an Item Revision is
        the output of the expansion, then its associated Datasets will be returned.
        """
        return cls.execute_soa_method(
            method_name='expandGRMRelationsForSecondary',
            library='Core',
            service_date='2007_06',
            service_name='DataManagement',
            params={'secondaryObjects': secondaryObjects, 'pref': pref},
            response_cls=ExpandGRMRelationsResponse,
        )

    @classmethod
    def getAvailableTypes(cls, classes: List[BaseClassInput]) -> GetAvailableTypesResponse:
        """
        This will return type names implemented by the given classes. This is lightweight way to get all displayable
        types by name rather than model object.
        """
        return cls.execute_soa_method(
            method_name='getAvailableTypes',
            library='Core',
            service_date='2007_06',
            service_name='DataManagement',
            params={'classes': classes},
            response_cls=GetAvailableTypesResponse,
        )

    @classmethod
    def getDatasetTypeInfo(cls, datasetTypeNames: List[str]) -> DatasetTypeInfoResponse:
        """
        This operation returns the named reference information for a set of dataset types.  Named references are
        Teamcenter objects that relate to a specific data file.
        
        Any failure that occurs during this operation is returned in the 'ServiceData' list of partial errors with the
        dataset type name string mapped to error message.
        
        
        Use cases:
        User wants to see which file type is allowed for attaching to a dataset.
        
        For this operation, the dataset type name is passed in the 'datasetTypeNames' input and the named reference
        information is returned.  The file extension, 'fileExtension', is returned in 'ReferenceInfo' and can be used
        to determine the supported file type for the dataset.
        """
        return cls.execute_soa_method(
            method_name='getDatasetTypeInfo',
            library='Core',
            service_date='2007_06',
            service_name='DataManagement',
            params={'datasetTypeNames': datasetTypeNames},
            response_cls=DatasetTypeInfoResponse,
        )


class SessionService(TcService):

    @classmethod
    def refreshPOMCachePerRequest(cls, refresh: bool) -> bool:
        """
        By Default the service operations will retrieve property value data straight from the POM. When 'refresh' is
        set to true, a refresh will be done on business objects before getting any property data. This will update the
        POM with fresh data from the database. The refresh is only applied to business objects that are actually being
        returned by a service operation. This applies only to database objects, and is not applied to runtime objects. 
        This is applied to all subsequent service requests from the same client. If multiple clients are sharing the
        same Teamcenter server session the refresh POM state is applied per client. Setting this to true will have a
        performance impact but will grantee all property values returned are up-to-date.
        """
        return cls.execute_soa_method(
            method_name='refreshPOMCachePerRequest',
            library='Core',
            service_date='2007_06',
            service_name='Session',
            params={'refresh': refresh},
            response_cls=bool,
        )


class LOVService(TcService):

    @classmethod
    def getAttachedLOVs(cls, inputs: List[LOVInfo]) -> AttachedLOVsResponse:
        """
        Get attached LOV based on input type name and property names structure. The LOV Tag is returned in the response
        and service data.
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='getAttachedLOVs',
            library='Core',
            service_date='2007_06',
            service_name='LOV',
            params={'inputs': inputs},
            response_cls=AttachedLOVsResponse,
        )


class PropDescriptorService(TcService):

    @classmethod
    def getAttachedPropDescs(cls, inputs: List[PropDescInfo]) -> AttachedPropDescsResponse:
        """
        Get the attached property descriptor based on input type name and property names structure.
        """
        return cls.execute_soa_method(
            method_name='getAttachedPropDescs',
            library='Core',
            service_date='2007_06',
            service_name='PropDescriptor',
            params={'inputs': inputs},
            response_cls=AttachedPropDescsResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Query._2013_05.SavedQuery import SavedQueryProperties
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class SavedQueryService(TcService):

    @classmethod
    def createSavedQueries(cls, inputs: List[SavedQueryProperties]) -> ServiceData:
        """
        Creates a list of saved queries based on the input properties structure.
        """
        return cls.execute_soa_method(
            method_name='createSavedQueries',
            library='Query',
            service_date='2013_05',
            service_name='SavedQuery',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )



from tcsoa.gen.Bom._2008_06.services import StructureManagementService as imp0
from tcsoa.gen.Bom._2010_09.services import StructureManagementService as imp1
from tcsoa.utils import TcService


class StructureManagementService(TcService):
    addOrUpdateChildrenToParentLine = imp0.addOrUpdateChildrenToParentLine
    createBaseline = imp0.createBaseline
    getTraversedObjectsByRule = imp1.getTraversedObjectsByRule
    removeChildrenFromParentLine = imp0.removeChildrenFromParentLine
    verifyObjectCoverageByRule = imp1.verifyObjectCoverageByRule



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class InitializeJBTResponse(TcBaseObj):
    """
    This structure contains the information of the site database that the client is connecting to.
    
    :var siteDBInfo: The last six characters of an UID which uniquely identifies a site database.
    :var serviceData: Partial errors and client id.
    :var additionalInfo: The additional information about the Journal Based Testing environment, such as the template
    names installed in the database.
    """
    siteDBInfo: str = ''
    serviceData: ServiceData = None
    additionalInfo: AuxiliaryInfo = None


@dataclass
class AuxiliaryInfo(TcBaseObj):
    """
    Auxiliary information about the Journal Based Testing environment, such as the template names installed in the
    database.
    
    :var strToStrVectorMap: A map of string to list of strings.
    :var strToDblVectorMap: A map of string to list of doubles.
    :var strToIntVectorMap: A map of string to list of integers.
    :var strToDateVectorMap: A map of string to list of dates.
    :var strToObjVectorMap: A map of string to list of Business Objects.
    """
    strToStrVectorMap: StringToStringVectorMap = None
    strToDblVectorMap: StringToDoubleVectorMap = None
    strToIntVectorMap: StringToIntegerVectorMap = None
    strToDateVectorMap: StringToDateMap = None
    strToObjVectorMap: StringToObjectVectorMap = None


@dataclass
class PropertyData(TcBaseObj):
    """
    The PropertyData structure contains the property data of a single object.
    
    :var object: The object whose property values will be validated.
    :var properties: The list of expected property values.
    """
    object: BusinessObject = None
    properties: List[SinglePropertyData] = ()


@dataclass
class SinglePropertyData(TcBaseObj):
    """
    The SinglePropertyData data structure contains the array value of a single property.
    
    :var propertyName: The name of the property.
    :var value: The array value of a property. For example, if the property is bl_child_lines, its value will be stored
    in a SinglePropertyData object. Because the value of this property represents multiple child BOMLine objects, the
    vector in SinglePropertyData is used to store all the child BOMLine objects. For all non-string values (e.g., float
    value), the value field stores their string representations.
    """
    propertyName: str = ''
    value: List[str] = ()


"""
A map of string to list of dates.
"""
StringToDateMap = Dict[str, List[datetime]]


"""
A map of string to list of doubles.
"""
StringToDoubleVectorMap = Dict[str, List[float]]


"""
A map of string to list of integers.
"""
StringToIntegerVectorMap = Dict[str, List[int]]


"""
A map of string to list of Business Objects.
"""
StringToObjectVectorMap = Dict[str, List[BusinessObject]]


"""
A map of string to list of strings.
"""
StringToStringVectorMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Internal.Manufacturing._2015_10.StructureManagement import AdditionalInfo
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetFutureRevisionsIn(TcBaseObj):
    """
    Structure to provide input BOMLine.
    
    :var inputLine: The BOMLine of type Mfg0BvrProcess or MfgBvrOperation that future revisions will be searched for.
    :var additionalInfo: For future use.
    """
    inputLine: BusinessObject = None
    additionalInfo: AdditionalInfo = None


@dataclass
class GetFutureRevisionsInfo(TcBaseObj):
    """
    Collection of revision information.
    
    :var line: BOMLine associated with future revision information.
    :var revInfoList: List of future revision information.
    :var additionalInfo: For future use.
    """
    line: BusinessObject = None
    revInfoList: List[RevisionInfo] = ()
    additionalInfo: AdditionalInfo = None


@dataclass
class GetFutureRevisionsResponse(TcBaseObj):
    """
    Structure to return future revision information for a line.
    
    :var lineFutureRevisionInfoMap: A map (BOMLine/std::vector< GetFutureRevisionsInfo >) of input line to associated
    future revisions information.
    :var serviceData: Partial errors and future use.
    :var additionalInfo: Reserved for future use.   For example ,  key = "Transitions" and values = "10 July","20 Aug".
    """
    lineFutureRevisionInfoMap: LineFutureRevisionInfoMap = None
    serviceData: ServiceData = None
    additionalInfo: AdditionalInfo = None


@dataclass
class RevisionInfo(TcBaseObj):
    """
    Collection of revision information.
    
    :var futureRevision: The future revision Item Revision.
    :var futureRevStatusInfo: The list of associated status and effectivity information for futureRevision.
    :var additionalInfo: For future use.
    """
    futureRevision: BusinessObject = None
    futureRevStatusInfo: List[StatusInfo] = ()
    additionalInfo: AdditionalInfo = None


@dataclass
class StatusInfo(TcBaseObj):
    """
    Collection of Status information.
    
    :var status: The status object.
    :var futureRevEffectivityInfo: The list of associated effectivity information for status.
    """
    status: BusinessObject = None
    futureRevEffectivityInfo: List[EffectivityInfo] = ()


@dataclass
class EffectivityInfo(TcBaseObj):
    """
    A collection of effectivity information.
    
    :var effectivity: The effectivity object.
    :var ranges: A list of effectivity ranges.
    :var effectivityTransitionsMap: A map(std::string/std::vector<std::string>) correlating an effectivity range to a
    corresponding list of effectivity transitions within that range.
    """
    effectivity: BusinessObject = None
    ranges: List[str] = ()
    effectivityTransitionsMap: RangeTransitionsEffectivityMap = None


"""
Map of input BOMLine to associated future revisions information
"""
LineFutureRevisionInfoMap = Dict[BusinessObject, List[GetFutureRevisionsInfo]]


"""
A map correlating an effectivity range to a corresponding list of effectivity transitions within that range.
"""
RangeTransitionsEffectivityMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Fnd0ApprSchCriteriaFormAttr, SearchStructureContext, Fnd0ApprSchCriteriaScpAttr
from tcsoa.gen.Rdv._2010_09.ContextManagement import StructureCntxtObjectInfo, CreateSCOInputInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetICSClassNamesResponse(TcBaseObj):
    """
    This structure contains the ICS class names and error, if any, in 'ServiceData' object.
    
    :var icsClassNames: ICS Class names for the corresponding     ApprSearchCriteriaInClass objects provided     in
    input paramter.
    :var serviceData: Contains any exceptions if occurred while getting the class name of InClass Search Criteria
    object.
    """
    icsClassNames: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class UpdateSearchSCOInputInfo(TcBaseObj):
    """
    This structure contains the Search Structure Context Object, the 'StructureCntxtObjectInfo' object containing
    information to be updated, a flag to indicate if results should be stored or not and the scope bomlines.
    
    :var srchSCO: Search Structure Context Object that needs to be updated. This SCO object will be updated with the
    modified values.
    
    :var scoInfo: Object of 'StructureCntxtObjectInfo' structure. It internally holds all the details related to the
    search criteria and search results.
    :var resultStored: Describes whether the results are stored or not.
    :var nodes: It is a vector of all the scope bomlines used.
    """
    srchSCO: SearchStructureContext = None
    scoInfo: StructureCntxtObjectInfo = None
    resultStored: bool = False
    nodes: List[BusinessObject] = ()


@dataclass
class UpdateSearchSCOResponse(TcBaseObj):
    """
    This structure contains the list of updated SearchStructureContext object and error, if any, in 'ServiceData'
    object.
    
    :var srchSCO: Vector of object of SearchStructureContext updated with the supplied inputs.
    :var serviceData: Contains any exception that occurred during updating the SearchSCO.
    """
    srchSCO: List[SearchStructureContext] = ()
    serviceData: ServiceData = None


@dataclass
class CreateFormAttrSearchCriteriaInputInfo(TcBaseObj):
    """
    Specifies the full set of inputs required to create the Form Search Criteria object.
    
    :var relationtype: Type name of Relation with which Form is attached to an Item/Item Revision. (For example:
    References). The valid relation types can be fetched from BMIDE: Open BMIDE->Go to Business Objects View->Search
    for ImanRelation->Expand ImanRelation. The name of any object listed as child of ImanRelation, can be supplied.
    :var parentType: Boolean value indicating type of parent (true if the parent type is Item and false if the parent
    type is Item Revision)
    :var formType: Type name of Form being used for the search. The valid form type can be fetched from BMIDE: Open
    BMIDE->Go to Business Objects View->Search for Form. The name of any object listed as child of 'Form' can be
    supplied.
    :var logicalOpr: Logical operator. The only valid operator currently supported is 'AND'.
    :var propertyName: Name of the selected property on the form used for the search.
    :var mathOpr: Operator used for comparison. List of valid operators: 
    - 'EQ' - Equal
    - 'NE' - Not Equal
    - 'GT' - Greater Than
    - 'GE' - Greater Than or Equal
    - 'LT' - Less Than
    - 'LE' - Less Than or Equal
    - 'LIKE'
    - 'NOT LIKE'
    
    
    :var propertyType: Name of the type of property. The valid string values for this member are: 
    - 'BooleanType' - For Boolean Property
    - 'DateType' - For Date Property
    - 'IntegerType' - For Integer Property
    - 'StringType' - For String Property
    - 'DoubleType' - For Double Property
    - 'TagType' - For Tag Property
    
    
    :var values: List of string values used for the search. Each value specified here would be an appropriate value for
    the property specified in 'propertyName' parameter. These     values are passed in string format and converted back
    to their original format based on the 'propertyType' parameter specified above.
    """
    relationtype: str = ''
    parentType: bool = False
    formType: str = ''
    logicalOpr: str = ''
    propertyName: str = ''
    mathOpr: str = ''
    propertyType: str = ''
    values: List[str] = ()


@dataclass
class CreateFormAttrSearchCriteriaResponse(TcBaseObj):
    """
    Contains the list of newly created Fnd0ApprSchCriteriaFormAttr objects and 'ServiceData' object
    
    :var formAttrSchCriteria: List of newly created Fnd0ApprSchCriteriaFormAttr objects.
    :var serviceData: An object of 'ServiceData' which contains any exceptions if occurred during creation of Form
    attribute object.
    """
    formAttrSchCriteria: List[Fnd0ApprSchCriteriaFormAttr] = ()
    serviceData: ServiceData = None


@dataclass
class CreateSearchCriteriaScopeInfo(TcBaseObj):
    """
    This structure is used to store the list of scope BOMLine objects and the Apprearance Goup objects.
    
    :var nodes: List of all the scope BOMLine objects. If the search is performed on occurrence group object then this
    list will be empty.
    
    :var collections: It is a vector of BusinessObjects. If search is performed on one or more Occurrence group BOMLine
    objects then this list should have those occurrence group objects.
    """
    nodes: List[BusinessObject] = ()
    collections: List[BusinessObject] = ()


@dataclass
class CreateSearchCriteriaScpResponse(TcBaseObj):
    """
    This structure is used to store information about the newly created Fnd0ApprSchCriteriaScpAttr objects and
    'ServiceData' object after execution of the operation 'createSearchCriteriaScope'.
    
    :var searchCriteriaScpObjects: The list of objects of Fnd0ApprSchCriteriaScpAttr created by the
    'createSearchCriteriaScope' action.
    :var serviceData: An object of 'ServiceData' which contains any exceptions if occurred during the creation of Scope
    Search Criteria object
    """
    searchCriteriaScpObjects: List[Fnd0ApprSchCriteriaScpAttr] = ()
    serviceData: ServiceData = None


@dataclass
class CreateSearchSCOInputInfo(TcBaseObj):
    """
    This structure is used to store additional information about a SCO object such as the availability of results,
    option to specify if SCO is shared or not.
    
    :var scoInputInfo: It is an object of 'CreateSCOInputInfo' structure which contains name, description, search
    criteria information.
    :var resultStored: Specifies whether results are stored or not in the SearchSCO.
    :var isShared: Specifies whether the SearchSCO object will be shared with other users or not.
    :var nodes: List of all the scope bomlines used to store additional information in the SearchSCO.
    """
    scoInputInfo: CreateSCOInputInfo = None
    resultStored: bool = False
    isShared: bool = False
    nodes: List[BusinessObject] = ()


@dataclass
class CreateSearchSCOResponse(TcBaseObj):
    """
    The CreateSearchSCOResponse structure represents search structure context object created and the service data.
    
    :var srchSCO: The vector of objects of SearchStructureContext created by the 'createSearchSCO' action
    
    :var serviceData: An object of 'ServiceData' which contains any exceptions if occurred during creation of SearchSCO
    """
    srchSCO: List[SearchStructureContext] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Administration._2017_05.GroupManagement import AddChildGroupsToGroupStructure
from tcsoa.gen.Administration._2017_05.RoleManagement import AddRolesToGroupStructure, RoleGroupStructure
from tcsoa.gen.Administration._2017_05.UserManagement import UserRoleGroupStructure, UserManagementResponse, AddUsersAsGroupMembersStructure
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class GroupManagementService(TcService):

    @classmethod
    def addChildGroups(cls, childGroupsToGroupStructs: List[AddChildGroupsToGroupStructure]) -> ServiceData:
        """
        Adds new groups and existing groups as child groups to specified Group objects. If groups are new, then they
        will be created first before they are added.
        """
        return cls.execute_soa_method(
            method_name='addChildGroups',
            library='Administration',
            service_date='2017_05',
            service_name='GroupManagement',
            params={'childGroupsToGroupStructs': childGroupsToGroupStructs},
            response_cls=ServiceData,
        )


class UserManagementService(TcService):

    @classmethod
    def removeGroupMembers(cls, userRoleGroupStructs: List[UserRoleGroupStructure]) -> UserManagementResponse:
        """
        This operation removes group members for specified User objects under given Group objects with specified Role
        objects. This operation requires system administration privilege or group administration privilege. Input
        should not have null User, Group and Role objects.
        """
        return cls.execute_soa_method(
            method_name='removeGroupMembers',
            library='Administration',
            service_date='2017_05',
            service_name='UserManagement',
            params={'userRoleGroupStructs': userRoleGroupStructs},
            response_cls=UserManagementResponse,
        )

    @classmethod
    def addUsersAsGroupMembers(cls, userRoleGroupStructs: List[AddUsersAsGroupMembersStructure]) -> ServiceData:
        """
        Adds new Users and existing Users as Group Members under the given Group objects with specific Role object. If
        a User is new, it will be created first before it is added as GroupMember. This operation requires system
        administration privilege or Group administration privilege. Specified Role objects must be an existing Role in
        the Group.
        """
        return cls.execute_soa_method(
            method_name='addUsersAsGroupMembers',
            library='Administration',
            service_date='2017_05',
            service_name='UserManagement',
            params={'userRoleGroupStructs': userRoleGroupStructs},
            response_cls=ServiceData,
        )


class RoleManagementService(TcService):

    @classmethod
    def removeRolesFromGroup(cls, roleGroupStructs: List[RoleGroupStructure]) -> ServiceData:
        """
        This operation removes specified Role objects from specified Group objects.
        """
        return cls.execute_soa_method(
            method_name='removeRolesFromGroup',
            library='Administration',
            service_date='2017_05',
            service_name='RoleManagement',
            params={'roleGroupStructs': roleGroupStructs},
            response_cls=ServiceData,
        )

    @classmethod
    def addRolesToGroup(cls, roleGroupStructs: List[AddRolesToGroupStructure]) -> ServiceData:
        """
        Adds new Roles and existing Roles to the specified Group objects. If the roles are new, it will create them
        first before they are added.
        """
        return cls.execute_soa_method(
            method_name='addRolesToGroup',
            library='Administration',
            service_date='2017_05',
            service_name='RoleManagement',
            params={'roleGroupStructs': roleGroupStructs},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Internal.DocMgmtAw._2019_06.DocMgmt import MarkupResponse, MarkupProperties
from tcsoa.utils import TcService
from tcsoa.gen.BusinessObjects import WorkspaceObject


class DocMgmtService(TcService):

    @classmethod
    def processMarkups(cls, baseObject: WorkspaceObject, version: str, properties: MarkupProperties, markups: str) -> MarkupResponse:
        """
        Description:
        This operation processes the markups on the server. 
        Use Cases:
        - Load all the existing markups from the server.
        - Save the user created or modified markups to the server.
        
        """
        return cls.execute_soa_method(
            method_name='processMarkups',
            library='Internal-DocMgmtAw',
            service_date='2019_06',
            service_name='DocMgmt',
            params={'baseObject': baseObject, 'version': version, 'properties': properties, 'markups': markups},
            response_cls=MarkupResponse,
        )



from __future__ import annotations

from tcsoa.gen.Core._2008_06.DataManagement import GetItemAndRelatedObjectsResponse, CreateResponse, SaveAsNewItemResponse2, GetItemAndRelatedObjectsInfo, AddParticipantOutput, ReviseResponse2, CreateOrUpdateItemElementsResponse, SaveAsNewItemInfo, CreateOrUpdateRelationsInfo, ItemElementProperties, TypeAndItemRevInfo, GetNRPatternsWithCountersResponse, ConnectionProperties, Participants, NRAttachInfo, AddParticipantInfo, CreateIn, CreateOrUpdateRelationsResponse, ReviseInfo, CreateOrUpdateGDELinksResponse, DatasetProperties2, GetNextIdsResponse, GetRevNRAttachResponse, InfoForNextId, GDELinkInfo, DisplayableSubBOsResponse, BOWithExclusionIn, CreateConnectionsResponse
from tcsoa.gen.BusinessObjects import BusinessObject, User
from tcsoa.gen.Core._2006_03.DataManagement import CreateDatasetsResponse
from tcsoa.gen.Core._2008_06.PropDescriptor import CreateDescResponse
from tcsoa.gen.Core._2008_06.DispatcherManagement import CreateDispatcherRequestArgs, CreateDispatcherRequestResponse
from tcsoa.gen.Core._2008_06.StructureManagement import GetSecondariesOfInStructureAssociationResponse, RemoveInStructureAssociationsInfo, GetSecondariesOfInStructureAssociationInfo, GetPrimariesOfInStructureAssociationInfo, GetPrimariesOfInStructureAssociationResponse, CreateInStructureAssociationResponse, InStructureAssociationInfo, RemoveInStructureAssociationsResponse
from tcsoa.gen.Core._2008_06.Session import GetDisplayStringsResponse
from tcsoa.gen.Core._2008_06.ManagedRelations import GetManagedRelationResponse, GetManagedRelationInput
from typing import List
from tcsoa.gen.Core._2006_03.Session import LoginResponse
from tcsoa.gen.Common import ObjectPropertyPolicy
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def getRevNRAttachDetails(cls, typeAndItemRevInfos: List[TypeAndItemRevInfo]) -> GetRevNRAttachResponse:
        """
        This operation gets all the possible initial, secondary and supplemental revision next Ids for an ItemRevision
        along with the available formats and the set of excluded letters for revision. The Revision Type Name and the
        current revision are passed in the input typeAndItemRevInfos structure. The input for this operation contains a
        list of this structure. The return structure contains list of Initial Revision details, Secondary Revision
        details, Supplemental Revision details and exclude Skip letters along with the service data member.
        
        Use cases:
        This operation is used to get the next available options for revision id for a new or existing object. User can
        select one of the values returned by this operation and use as revision id input value for create, revise or
        save-as operation.
        """
        return cls.execute_soa_method(
            method_name='getRevNRAttachDetails',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'typeAndItemRevInfos': typeAndItemRevInfos},
            response_cls=GetRevNRAttachResponse,
        )

    @classmethod
    def removeParticipants(cls, participants: List[Participants]) -> ServiceData:
        """
        This operation allows the user to remove Participant objects associated with specified Item Revision. If a
        participant being removed is no longer associated with any other objects, it gets deleted.
        
        Use cases:
        This operation can be used to remove the assigned Participant objects like Analyst, Proposed Reviewers etc.
        from the change objects. Change creator can use 'addParticipants' service operation to assign an analyst or use
        this operation to remove an assigned analyst.
        """
        return cls.execute_soa_method(
            method_name='removeParticipants',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'participants': participants},
            response_cls=ServiceData,
        )

    @classmethod
    def revise2(cls, info: List[ReviseInfo]) -> ReviseResponse2:
        """
        This operation provides the ability to revise the ItemRevision objects given in the input list and carries
        forward relations to existing objects. When applying deep copy rules, if user overridden deep copy information
        is provided in the input, relations are propagated to the new ItemRevision based on that input. If no deep copy
        information is provided in the input, the deep copy rules in the database are used to propagate relations to
        the new ItemRevision.  If the input contains property values for the master form, those values are set on the
        new ItemRevision master form.
        """
        return cls.execute_soa_method(
            method_name='revise2',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'info': info},
            response_cls=ReviseResponse2,
        )

    @classmethod
    def saveAsNewItem2(cls, info: List[SaveAsNewItemInfo]) -> SaveAsNewItemResponse2:
        """
        This operation provides the capability to create one or more new Item objects based on a list of existing
        ItemRevision objects. It optionally carries forward ItemRevision relations based on the 'deepCopyRequired'
        flag. When applying deep copy rules, if user overridden deep copy information is provided in the input, then
        old ItemRevision relations are propagated to the new ItemRevision based on the input. If no deep copy rule
        information is provided in the input, the deep rules in the database will be applied. If user provides new
        property values for the Item and ItemRevision master forms in the input, then these will be copied to the
        master forms of the newly created Item and ItemRevision.
        """
        return cls.execute_soa_method(
            method_name='saveAsNewItem2',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'info': info},
            response_cls=SaveAsNewItemResponse2,
        )

    @classmethod
    def createConnections(cls, properties: List[ConnectionProperties], container: BusinessObject, relationType: str) -> CreateConnectionsResponse:
        """
        Creates a list of Connection objects and any associated data (ConnectionRevision, ConnectionMaster Form and
        ConnectionRevision Master Form) based on the input properties structure. It also creates the specified relation
        type between newly created Connection object and input container object. If container and relation type are not
        supplied, none of the Connection objects will be related to a container. If any destination to paste the newly
        created objects is desired then it must be supplied as input.
        
        Use cases:
        This operation supports creation of single or multiple Connection objects.
        """
        return cls.execute_soa_method(
            method_name='createConnections',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'properties': properties, 'container': container, 'relationType': relationType},
            response_cls=CreateConnectionsResponse,
        )

    @classmethod
    def createDatasets2(cls, input: List[DatasetProperties2]) -> CreateDatasetsResponse:
        """
        This operation creates a list of Dataset objects and creates the specified relation type between created
        Dataset and input container object.
        """
        return cls.execute_soa_method(
            method_name='createDatasets2',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreateDatasetsResponse,
        )

    @classmethod
    def createObjects(cls, input: List[CreateIn]) -> CreateResponse:
        """
        Generic operation for creation of Business Objects. This will also create any secondary(compounded) objects
        that need to be created, assuming the CreateInput for the secondary object is represented in the recursive
        CreateInput object e.g. Item is Primary Object that also creates Item Master and ItemRevision. ItemRevision in
        turn creates ItemRevision Master. The input for all these levels is passed in through the recursive CreateInput
        object.
        
        Use cases:
        This operation to create an object is invoked after obtaining user input on the fields of the create dialog.
        This call is typically preceded by a call to Teamcenter::Soa::Core::_2008_06::PropDescriptor::getCreateDesc or
        to the SOA Client Metamodel layer to retrieve Create Descriptor for a Business Object. 
        
        For example, to create an Item, client will get the Create Descriptor associated with the Item from the client
        metamodel (The associated descriptor type can be found by looking at the constant value for the CreateInput
        constant that is attached to Item). Alternatively, for clients that do not use the SOA client metamodel, the
        Descriptor for Item can be obtained by invoking getCreateDesc operation. The descriptor information can then be
        used to populate the Create dialog for the Business Object. Once the Create dialog is populated the
        createObjects operation can be called to create the object.
        """
        return cls.execute_soa_method(
            method_name='createObjects',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreateResponse,
        )

    @classmethod
    def createOrUpdateGDELinks(cls, gdeLinkInfos: List[GDELinkInfo]) -> CreateOrUpdateGDELinksResponse:
        """
        Create and/or update GeneralDesignElementLink(GDELink) objects based on the input properties structure. If the
        given GDELink object exists in Teamcenter then the operation will be treated as an update based on the input
        properties structure
        
        Use cases:
        This operation supports creation or updation of GDELink objects
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateGDELinks',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'gdeLinkInfos': gdeLinkInfos},
            response_cls=CreateOrUpdateGDELinksResponse,
        )

    @classmethod
    def createOrUpdateItemElements(cls, properties: List[ItemElementProperties]) -> CreateOrUpdateItemElementsResponse:
        """
        Allows the user to create a new GeneralDesignElement (GDE) or its subtype and set its properties. In the case
        of existing GDE, user can update the properties.
        
        Use cases:
        This operation can be used when user wants to create a GDE in a product structure or wants to update the
        properties of an existing GDE. User has to pass unique client Id, name and type when a new element has to be
        created. Whenever properties of an existing GDE have to be updated the itemElement business object and
        itemElemAttributes should be passed
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateItemElements',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'properties': properties},
            response_cls=CreateOrUpdateItemElementsResponse,
        )

    @classmethod
    def createOrUpdateRelations(cls, infos: List[CreateOrUpdateRelationsInfo], sync: bool) -> CreateOrUpdateRelationsResponse:
        """
        Creates the specified relation between the input objects (primary and secondary objects) for each input
        structure. If the 'sync' flag is specified, then if any Generic Relationship Management (GRM) relations exist
        between the primary and secondary objects and they are not specified in the input they will be deleted. If
        'sync' flag is provided, the relations member of 'CreateOrUpdateRelationsInfo' is ignored.
        
        Use cases:
        Use Case 1: Create a relation based on the GRM rule definition.
        
        One can create a relation specified by name of the relation in the input structure between the primary and
        secondary object, when there exists a GRM rule between the primary and secondary object with the given relation
        type.
        
        Use Case 2: Update a relation based on the GRM rule definition.
        
        One can update a relation specified by name of the relation in the input structure between the primary and
        secondary object, when there exists a GRM rule between the primary and secondary object with the given relation
        type.
        
        Use Case 3: Remove a relation based on the GRM rule definition.
        
        One can remove a relation by setting the sync to true and do not provide any relation between the primary and
        secondary object in the input structure. When there exists a GRM rule between the primary and secondary object
        with the given relation type, and the sync flag is set to true, then if any GRM relations exist between the
        primary and secondary objects and they are not specified in the input they will be deleted.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateRelations',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'infos': infos, 'sync': sync},
            response_cls=CreateOrUpdateRelationsResponse,
        )

    @classmethod
    def addParticipants(cls, addParticipantInfo: List[AddParticipantInfo]) -> AddParticipantOutput:
        """
        This operation creates the Participant objects and adds them to the input Item Revision. If a Participant
        object with specified attributes already exists, it is added to the Item Revision.
        
        Use cases:
        For Change Management use cases, user may need to add different participants to the change objects like
        analyst, change specialist etc. The creator of the change object will assign the analyst for the change where
        this operation can be used.
        """
        return cls.execute_soa_method(
            method_name='addParticipants',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'addParticipantInfo': addParticipantInfo},
            response_cls=AddParticipantOutput,
        )

    @classmethod
    def findDisplayableSubBusinessObjects(cls, input: List[BOWithExclusionIn]) -> DisplayableSubBOsResponse:
        """
        Operation to retrieve sub Business Object Names for a Business Object that are displayable to the login user in
        the object creation dialog. e.g File-new, select Item, what types to be displayed for Item
        """
        return cls.execute_soa_method(
            method_name='findDisplayableSubBusinessObjects',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=DisplayableSubBOsResponse,
        )

    @classmethod
    def getItemAndRelatedObjects(cls, infos: List[GetItemAndRelatedObjectsInfo]) -> GetItemAndRelatedObjectsResponse:
        """
        This operation returns Items, ItemRevisions, Dataset and NamedReference information based on the input. Input
        is a list of 'GetItemAndRelatedObjectsInfo' structures each of which contains item uid or id, revison
        information and optionally a list of filters to determine the datasets to be returned. For the Datasets the
        client can request information about the NamedReferences. NamedReferences are how Data files are attached to
        Datasets. The Data file is what a CAD client is really interested in. The return is a
        'GetItemAndRelatedObjectsResponse' which contains a list of 'GetItemAndRelatedObjectsItemOutput' and a
        'ServiceData'.
        
        Use cases:
        The client has an item of ItemRevision and needs to know what CAD data is attached to the ItemRevision. The
        client fills in either the Item or ItemRevision information along with the information about the types of
        Dataset and NamedReferences the client is interested in. For the NamedReferences the user can get the tickets
        which will allow the client retrieve the files attached to the Datasets. The results of the query will give the
        client all the information about the Datasets and NamedReferences and optional tickets.
        """
        return cls.execute_soa_method(
            method_name='getItemAndRelatedObjects',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'infos': infos},
            response_cls=GetItemAndRelatedObjectsResponse,
        )

    @classmethod
    def getNRPatternsWithCounters(cls, vAttachInfo: List[NRAttachInfo]) -> GetNRPatternsWithCountersResponse:
        """
        This operation gives the list of Patterns which has counters along with preferred patterns and conditions for
        the Naming Rule attached to the property of an object Type. The Type name and the Property name are passed in
        the input structure. The input for this operation contains a list of this structure. The return structure
        contains the vector of patterns with counters, preferredPattern and condition, along with the service data
        member.
        
        Use cases:
        This operation is used in create, revise or save-as dialogs to receive list of applicable patterns for item and
        revision ids. The list of patterns returned is displayed in these dialogs for user selection. After user
        selects a pattern and clicks "Assign" button, an id is generated matching the pattern selected and populated in
        the corresponding field's box.
        """
        return cls.execute_soa_method(
            method_name='getNRPatternsWithCounters',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'vAttachInfo': vAttachInfo},
            response_cls=GetNRPatternsWithCountersResponse,
        )

    @classmethod
    def getNextIds(cls, vInfoForNextId: List[InfoForNextId]) -> GetNextIdsResponse:
        """
        The operation generates the next id as per the input pattern in the attached Naming Rule to the property of an
        object type. The type name, property name and pattern are passed in the input structure. The input for this
        operation contains a  list of this structure. The return structure contains the list of nextId along with the
        service data member. In the case where no pattern is specified, a default ID will be returned if the type,
        property combination is Item / item_id, Dataset / pubr_object_id or ItemRevision/item_revision_id. For all
        other type, property combinations, the operation will not generate a default ID.
        
        Use cases:
        This operation is called on click of "Assign" button in create, revise and save-as dialogs to generate the next
        available options for item and revision ids. The generated values are used as input for create, revise and
        save-as operations.
        """
        return cls.execute_soa_method(
            method_name='getNextIds',
            library='Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'vInfoForNextId': vInfoForNextId},
            response_cls=GetNextIdsResponse,
        )


class StructureManagementService(TcService):

    @classmethod
    def getSecondariesOfInStructureAssociation(cls, inputs: List[GetSecondariesOfInStructureAssociationInfo]) -> GetSecondariesOfInStructureAssociationResponse:
        """
        Given the primary object and association type, returns the secondary BOMLine business objects of in structure
        associations. These associations can be ConnectTo, ImplementedBy, RealizedBy, RoutedBy, FixingToSegment,
        DeviceToConnector, SignalToSource, SignalToTarget, SignalToTransmitter, ProcessVariable or RedundantSignal. It
        takes a vector of GetSecondariesOfInStructureAssociationInfo as input.
        
        Use cases:
        Users shall use this operation to get secondary BOMLine business objects for a given association type and the
        primary object associated with it. 
        For instance, this operation could be used to get all the secondary GDE lines connected to a PSConnection by
        passing the association type as ConnectTo. Similarly, it can be used to get the connector lines connected to
        the Device Line by passing appropriate primary Device and the association type as DeviceToConnector.
        """
        return cls.execute_soa_method(
            method_name='getSecondariesOfInStructureAssociation',
            library='Core',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'inputs': inputs},
            response_cls=GetSecondariesOfInStructureAssociationResponse,
        )

    @classmethod
    def removeInStructureAssociations(cls, inputs: List[RemoveInStructureAssociationsInfo]) -> RemoveInStructureAssociationsResponse:
        """
        Given the primary BOMLine, the association type, and the secondary BOMLines this operation removes the
        instructure associations between the BOMLines. These associations can be ConnectTo, ImplementedBy, RealizedBy,
        RoutedBy, FixingToSegment, DeviceToConnector, SignalToSource, SignalToTarget, SignalToTransmitter,
        ProcessVariable or RedundantSignal. The operation takes a vector of RemoveInStructureAssociationsInfo as Input.
        If input primary is Signal object's BOMLine, then for associatiion type input between signal and secondary as
        source or target can be optional, and if input association type is empty, then the secondary BOMLine
        association to input primary Signal BOMLine  as source and target will be removed.
        
        Use cases:
        Developers shall use this operation when an association has to be removed between the BOMLines. This is a
        generic Teamcenter service to remove various types of associations. 
        In the case of ConnectTo, if the secondary BOMLines are passed as null then all the secondary associations with
        the primary BOMLine shall be removed.
        """
        return cls.execute_soa_method(
            method_name='removeInStructureAssociations',
            library='Core',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'inputs': inputs},
            response_cls=RemoveInStructureAssociationsResponse,
        )

    @classmethod
    def createInStructureAssociations(cls, inputs: List[InStructureAssociationInfo]) -> CreateInStructureAssociationResponse:
        """
        This operation creates the specified association between primary and secondary BOMLine objects in a structure. 
        As the name indicates, these associations are created in a specific context and are applicable only in that
        context. The context is specified as an additional input in the input structure, by the caller. Some examples
        of these associations are: the ConnectTo, ImplementedBy, RealizedBy, SignalToSource, SignalToTarget,
        SignalToTransmitter, ProcessVariable, RedundantSignal relations that are provided in Teamcenter [see Use case].
        This operation takes a vector of InStructureAssociationInfo as input. The input structures contain information
        on the BOMLine objects that need to be associated, what context they need to be associated in and the type of
        association. Note that the associations created are only valid in the context specified.
        If input primary is Signal object's BOMLine, then for associating signal BOMLine to secondary as source or
        target, the association type is optional, and if input association type is empty, the secondary BOMLine objects
        GDE object direction attirbute value will be used for associating signal BOMLine to secondary as source or
        target.
        
        Use cases:
        Use this operation to create an association between BOMLine objects. This is a generic Teamcenter service to
        create various types of associations. The type of the association that gets created depends on the Association
        Type specified in the input structure. 
        The ConnectTo  functionality establishes an association between 1 or more BOMLine objects and a Connection
        BOMLine. The association signifies that the BOMLine objects are connected to the Connection BOMLine in a
        certain context.  Outside of that context the association is not valid. The caller of this operation provides
        as input, one or more BOMLine objects, and a Connection BOMLine. The associationType should be set in the input
        structure to ConnectTo and specifies a BOMLine that will act as a context line within which the association is
        valid.
        """
        return cls.execute_soa_method(
            method_name='createInStructureAssociations',
            library='Core',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'inputs': inputs},
            response_cls=CreateInStructureAssociationResponse,
        )

    @classmethod
    def getPrimariesOfInStructureAssociation(cls, inputs: List[GetPrimariesOfInStructureAssociationInfo]) -> GetPrimariesOfInStructureAssociationResponse:
        """
        This operation gets the primary BOMLines like PSConnection, Signal of an association given the secondary object
        and association type.  Examples of these associations are: ConnectTo, ImplementedBy, RealizedBy, RoutedBy,
        FixingToSegment, DeviceToConnector, SignalToTransmitter, SignalToSource, SignalToTarget, ProcessVariable,
        RedundantSignal. This operation takes a vector of GetPrimariesOfInStructureAssociationInfo as input.
        
        Use cases:
        A typical usecase for this operation is where callers would like to obtain primary BOMLine objects by providing
        the association type and the corresponding secondary BOMLine associated.  In a SignalToTransmitter association
        for example, if the transmitting BOMLine  is provided, callers can get the corresponding Signal which is the
        primary BOMLine in this relation.
        """
        return cls.execute_soa_method(
            method_name='getPrimariesOfInStructureAssociation',
            library='Core',
            service_date='2008_06',
            service_name='StructureManagement',
            params={'inputs': inputs},
            response_cls=GetPrimariesOfInStructureAssociationResponse,
        )


class SessionService(TcService):

    @classmethod
    def login(cls, username: str, password: str, group: str, role: str, locale: str, sessionDiscriminator: str) -> LoginResponse:
        """
        Authenticates the user`s credentials and initialize a Teamcenter session for this client. The operation will
        throw an 'InvalidCredentialsException' if the 'username', 'password' or 'group' is not valid.
        When the client application is deployed to a 4Tier environment (communication through HTTP or TCCS) the 'login'
        operation also contributes to the assignment of a Teamcenter server instance to the client session. The
        Teamcenter architecture varies from other client server architectures in that there is a dedicated instance of
        the Teamcenter server per client application. However, there are use cases where it is desirable for a single
        user to have multiple desktop applications running and each sharing a single instance of a Teamcenter server.
        This is controlled through the following elements:
        - 'hostPath'    From the Connection class constructor, this specifies  the address (URI) the Teamcenter server
        is hosted on.
        - 'username'    From this login operation, this specifies the user`s Teamcenter user name.
        - 'sessionDiscriminator'    From this login operation, this identifies the client session.
        
        
        
        The 'hostPath' argument determines the server machine that the client connects to. Once there, the pool manager
        on that host uses the 'username' and 'sessionDiscriminator' arguments of the 'logon' request to determine which
        Teamcenter server instance to assign the client to. If the pool manager has an existing Teamcenter server
        instance with the 'username'/'sessionDiscriminator' key, the client is assigned to that existing instance of
        the Teamcenter server, and therefore sharing the server with another client; otherwise, a new instance of the
        Teamcenter server is used. There are a few general scenarios for the 'sessionDiscriminator' argument:
        
        - Blank     If the user jdoe logs on to Teamcenter using two or more client applications using a blank
        'sessionDiscriminator' argument (for example, jdoe/ ), all of those clients are assigned to the same Teamcenter
        server instance. These client applications can be running on the same or different client hosts.
        - Constant     If the user jdoe logs on to Teamcenter using two or more client applications using a constant or
        fixed 'sessionDiscriminator' argument (for example, jdoe/MyApp ), those clients are assigned to the same
        Teamcenter server instance. This is similar to the blank 'sessionDiscriminator' argument; the difference is
        that only multiple instances of the client application using myApp started by jdoe share the same Teamcenter
        server instance.
        - Unique     If the user jdoe logs on using a unique random generated string (for example, jdoe/akdk938lakc),
        the client application will be assigned to a dedicated instance of the Teamcenter server.
        
        
         
        The scenario you use depends on how your client application is used in the integrated environment. The most
        common case is the unique 'sessionDiscriminator' value.
        
        Exceptions:
        >When the credentials supplied are invalid or the requested locale is not allowed.  
        - 515143:    The logon was refused due to invalid 'username' or 'password'
        - 515144:    The logon was refused due to invalid 'username' or 'password'
        - 515142:    The logon was refused due to an invalid 'group'.
        - 128001: The logon was refused due to conflict with the encoding of the database instance. 
        - 128002:    The logon was refused due to missing localization.
        
        """
        return cls.execute_soa_method(
            method_name='login',
            library='Core',
            service_date='2008_06',
            service_name='Session',
            params={'username': username, 'password': password, 'group': group, 'role': role, 'locale': locale, 'sessionDiscriminator': sessionDiscriminator},
            response_cls=LoginResponse,
        )

    @classmethod
    def loginSSO(cls, username: str, ssoCredentials: str, group: str, role: str, locale: str, sessionDiscriminator: str) -> LoginResponse:
        """
        Authenticates the user using Single-Sign-On (SSO) credentials and initialize a Teamcenter session for this
        client. The 'username' and 'ssoCredentials' arguments must be obtained from Teamcenter's Security Services. The
        'SsoCredentials' class offers a simple API to get these values. The operation will throw an
        'InvalidCredentialsException' if the 'username', 'ssoCredentials' or 'group' is not valid.
        When the client application is deployed to a 4Tier environment (communication through HTTP or TCCS) the login
        operation also contributes to the assignment of a Teamcenter server instance to the client session. The
        Teamcenter architecture varies from other client server architectures in that there is a dedicated instance of
        the Teamcenter server per client application. However, there are use cases where it is desirable for a single
        user to have multiple desktop applications running and each sharing a single instance of a Teamcenter server.
        This is controlled through the following elements:
        'hostPath'    From the Connection class constructor, this specifies  the address (URI) the Teamcenter server is
        hosted on.
        'username'    From this login operation, this specifies the user's Teamcenter user name.
        'sessionDiscriminator'    From this login operation, this identifies the client session.
        
        The 'hostPath' argument determines the server machine that the client connects to. Once there, the pool manager
        on that host uses the 'username' and 'sessionDiscriminator' arguments of the login request to determine which
        Teamcenter server instance to assign the client to. If the pool manager has an existing Teamcenter server
        instance with the 'username'/'sessionDiscriminator' key, the client is assigned to that existing instance of
        the Teamcenter server, and therefore sharing the server with another client; otherwise, a new instance of the
        Teamcenter server is used. There are a few general scenarios for the 'sessionDiscriminator' argument:
        
        - Blank     If the user jdoe logs on to Teamcenter using two or more client applications using a blank
        sessionDiscriminator argument (for example, jdoe/ ), all of those clients are assigned to the same Teamcenter
        server instance. These client applications can be running on the same or different client hosts.
        - 'Constant'     If the user jdoe logs on to Teamcenter using two or more client applications using a constant
        or fixed sessionDiscriminator argument (for example, jdoe/MyApp ), those clients are assigned to the same
        Teamcenter server instance. This is similar to the blank 'sessionDiscriminator' argument; the difference is
        that only multiple instances of the client application using myApp started by jdoe share the same Teamcenter
        server instance.
        - Unique     If the user jdoe logs on using a unique random-generated string (for example, jdoe/akdk938lakc),
        the client application will be assigned to a dedicated instance of the Teamcenter server.
        
        
         
        The scenario you use depends on how your client application is used in the integrated environment. The most
        common case is the unique 'sessionDiscriminator' value.
        
        Exceptions:
        >When the credentials supplied are invalid or the requested locale is not allowed.  
        - 515143:    The logon was refused due to invalid 'username' or 'password'
        - 515144:    The logon was refused due to invalid 'username' or 'password'
        - 515142:    The logon was refused due to an invalid 'group'.
        - 128001: The logon was refused due to conflict with the encoding of the database instance.
        
        """
        return cls.execute_soa_method(
            method_name='loginSSO',
            library='Core',
            service_date='2008_06',
            service_name='Session',
            params={'username': username, 'ssoCredentials': ssoCredentials, 'group': group, 'role': role, 'locale': locale, 'sessionDiscriminator': sessionDiscriminator},
            response_cls=LoginResponse,
        )

    @classmethod
    def setObjectPropertyPolicy(cls, policy: ObjectPropertyPolicy) -> str:
        """
        Sets the current object property policy. The business logic of a service operation determines what business
        objects are returned, while the object property policy determines which properties are returned on each
        business object instance.  This allows the client application to determine how much or how little data is
        returned based on how the client application uses those returned business objects. The policy is applied
        uniformly to all service operations. 
        By default, all applications use the Default object property policy, defined on the Teamcenter server
        '$TC_DATA/soa/policies/default.xml.' It is this policy that is applied to all service operation responses until
        the client application changes the policy. Siemens PLM Software strongly recommends that all applications
        change the policy to one applicable to the client early in the session.
        The object property policy will stay in affect for this session until changed by another call to
        'setObjectPRopertyPolicy'. The current policy can be modified with calls to 'updatObjectPropertyPolicy'.
        
        Like any other service operation, this operation cannot be called before establishing a session with the
        'login' serivce operation, so if you need a policy other than the Default policy for the business objects
        returned by the 'login' operation, use the _2011_06 version of the' login/loginSso' operation to authenticate
        and establish a session without returning business objects. The 'setObjectPropertyPolicy' operation can then be
        called to establish the policy for the session.
        """
        return cls.execute_soa_method(
            method_name='setObjectPropertyPolicy',
            library='Core',
            service_date='2008_06',
            service_name='Session',
            params={'policy': policy},
            response_cls=str,
        )

    @classmethod
    def getDisplayStrings(cls, info: List[str]) -> GetDisplayStringsResponse:
        """
        Get the localized text string for each input key from the info array. The input key must correspond to a key
        defined in the Text Server. If the input array is empty, the returned array will also be empty.
        """
        return cls.execute_soa_method(
            method_name='getDisplayStrings',
            library='Core',
            service_date='2008_06',
            service_name='Session',
            params={'info': info},
            response_cls=GetDisplayStringsResponse,
        )


class ReservationService(TcService):

    @classmethod
    def transferCheckout(cls, objects: List[BusinessObject], userId: User) -> ServiceData:
        """
        This operation transfers the previously checked-out business objects to the user given from input. This
        operation takes care of all complex business logic involved in transfer checked-out based on passed in objects.
        Each input object is verified that it is valid for transferring its checkout. 
        This operation validates precondition defined per type in COTS object and site customization before performing
        transfer checked-out objects to the target user. Dataset, ItemRevision and many other business object types
        have their own business logic for transfer check-out. This operation calls underlying transfer checkout method
        of those individual objects.
        """
        return cls.execute_soa_method(
            method_name='transferCheckout',
            library='Core',
            service_date='2008_06',
            service_name='Reservation',
            params={'objects': objects, 'userId': userId},
            response_cls=ServiceData,
        )


class DispatcherManagementService(TcService):

    @classmethod
    def createDispatcherRequest(cls, inputs: List[CreateDispatcherRequestArgs]) -> CreateDispatcherRequestResponse:
        """
        Create a DispatcherRequest within Teamcenter for use with Dispatcher Management Services.
        
        Use cases:
        The Dispatcher Management application provides the ability to process requests in an asynchronous fashion thus
        removing the processing burden from the clients to provisioned machine dedicated to processing these requests. 
        There are quite a few services, within Teamcenter and other applications that use this application.
        
        Here are a few examples:
        - Asynchronous Processing (if configured)
        - NXtoPVDirect (provided with NX)
        - PreviewService
        
        """
        return cls.execute_soa_method(
            method_name='createDispatcherRequest',
            library='Core',
            service_date='2008_06',
            service_name='DispatcherManagement',
            params={'inputs': inputs},
            response_cls=CreateDispatcherRequestResponse,
        )


class PropDescriptorService(TcService):

    @classmethod
    def getCreateDesc(cls, businessObjectTypeNames: List[str]) -> CreateDescResponse:
        """
        The operation returns information required to create a Business Object. The Create Descriptor (CreateDesc
        object) includes the metadata information for the properties required when creating a business object  i.e,
        property is mandatory, property is visible, etc. The CreateDesc is a recursive data structure. The CreateDesc
        object can also reference CreateDesc on secondary objects through a reference or relation property. For
        example, the CreateDesc for Item points to CreateDesc on its related secondary objects, ItemRevison and Item
        Master. The CreateDesc for ItemRevision in turn points to the CreateDesc for ItemRevision Master.
        
        NOTE:  The operation will be deprecated as of Teamcenter 10. All the metadata information necessary for the
        Create operation can be retrieved from the SOA client metamodel.
        
        
        Use cases:
        Get Create Descriptor information for the Create wizard for an object.
        This call is made to populate the Create dialog for any Business Object. After obtaining the user input on the
        fields of the create dialog, a call is made to the
        'Teamcenter::Soa::Core::_2008_06::DataManagement::createObjects' operation to create the object
        """
        return cls.execute_soa_method(
            method_name='getCreateDesc',
            library='Core',
            service_date='2008_06',
            service_name='PropDescriptor',
            params={'businessObjectTypeNames': businessObjectTypeNames},
            response_cls=CreateDescResponse,
        )


class ManagedRelationsService(TcService):

    @classmethod
    def getManagedRelations(cls, inputdata: GetManagedRelationInput) -> GetManagedRelationResponse:
        """
        This operation will return tracelinks between primary and secondary objects
        """
        return cls.execute_soa_method(
            method_name='getManagedRelations',
            library='Core',
            service_date='2008_06',
            service_name='ManagedRelations',
            params={'inputdata': inputdata},
            response_cls=GetManagedRelationResponse,
        )



from __future__ import annotations

from tcsoa.utils import TcBaseObj
from tcsoa.gen.BusinessObjects import WorkspaceObject
from dataclasses import dataclass


@dataclass
class GetContentInput1(TcBaseObj):
    """
    'GetContentInput1' structure represents the parameters required to open requirement in MSWord.
    
    :var objectToProcess: Fulltext object that user want to view.
    :var applicationFormat: For viewing and editing content without template this parameter should be MSWordXML.
    For viewing and editing object by applying template this parameter should be MSWordXMLLive.
    
    
    :var templateId: This parameter is not used currently.
    :var applyTemplates: For viewing and editing content without template this parameter should be false.
    For viewing and editing object by applying template this parameter should be true.
    """
    objectToProcess: WorkspaceObject = None
    applicationFormat: str = ''
    templateId: str = ''
    applyTemplates: bool = False



from __future__ import annotations

from tcsoa.gen.Internal.StructureManagement._2007_12.BrokenLinks import RepairBrokenLinksParam, FindAndFixInput, FindCandidatesResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class BrokenLinksService(TcService):

    @classmethod
    def repairBrokenLinks(cls, input: List[RepairBrokenLinksParam]) -> ServiceData:
        """
        Repair the broken links by linking the broken link with the candidate.
        
        Use cases:
        User has a big product structure, and many assemblies are consumed in process structure. The product structure
        is restructured, so quite some links are showing as broken. User invokes the operation
        'getBrokenLinkInfoWithFixOpt' and finds all candidates for the broken links. Some broken links have more than
        one candidates, user selects one candidate for each broken link and fixes them by invoking this operation.
        """
        return cls.execute_soa_method(
            method_name='repairBrokenLinks',
            library='Internal-StructureManagement',
            service_date='2007_12',
            service_name='BrokenLinks',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def getBrokenLinkInfoWithFixOpt(cls, input: List[FindAndFixInput]) -> FindCandidatesResponse:
        """
        For manufacturing, links are built between structures, for example, product structures and process structures.
        Sometimes, a link can be broken due to structure changes. This operation finds broken links and candidates for
        repair with fix option.
        
        1.    If broken link scope is passed in, find the broken link in the scope (quick or thorough)
        2.    If search criteria is not empty, find the candidates for the broken links
        - Most commonly used criteria is item id.
        - Property "'bl_abs_occ_id'", "'bl_usage_address'", "'bl_position_designator'" and any occurrence note will not
        require revision rule. Other property will require revision rule to find candidates.
        - Properties can be mandatory or optional for matching. All mandatory properties and at least one optional
        property should match for a line to qualify as a candidate to repair a broken link.
        
        
        3.    If automatic fix is enabled and one-to-one match is found, fix the broken links
        
        Use cases:
        - User has a linked structure, but the source is deleted by accident. User adds it back and calls the operation
        to repair the broken link by passing the broken link, the root of the product structure, and option to
        automatic fix, the broken link will be automatically fixed by using the added back line.
        - User has a big product structure, and many assemblies are consumed in process structure. The product
        structure is restructured, so quite some links are showing as broken. User invokes this operation by using the
        root links of the two structures, item id, "'bl_abs_occ_id'", "'bl_usage_address'", "'bl_position_designator'"
        as mandatory criteria and does a thorough search. It will return all candidates for the broken links. Some
        broken links have more than one candidates, user selects one candidate for each broken link and fixes them by
        invoking 'repairBrokenLinks' operation.
        
        """
        return cls.execute_soa_method(
            method_name='getBrokenLinkInfoWithFixOpt',
            library='Internal-StructureManagement',
            service_date='2007_12',
            service_name='BrokenLinks',
            params={'input': input},
            response_cls=FindCandidatesResponse,
        )



from tcsoa.gen.Internal_Gdis._2006_03.services import ReportingService as imp0
from tcsoa.utils import TcService


class ReportingService(TcService):
    getPrecalRoutineInfo = imp0.getPrecalRoutineInfo



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetSubTypeHierarchicalTreeInput(TcBaseObj):
    """
    The main input here is the internal name of given type.
    
    :var typeInternalName: Internal name of type being requested.
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    """
    typeInternalName: str = ''
    clientId: str = ''


@dataclass
class GetSubTypeHierarchicalTreeResponse(TcBaseObj):
    """
    Holds linked type hierarchical tree data for requested types.  The structured trees start from given input types
    and consist of all level sub types.
    
    :var treeRootNodes: Root nodes of type hierachical trees. Each root node holds internal name and display name of
    given input type as well as its all immidiate sub types ordered by display name. Each immidiate sub type node holds
    its immidiate sub types ordered by display name.
    :var serviceData: Returned service data.
    """
    treeRootNodes: List[TypeHierarchicalTreeNode] = ()
    serviceData: ServiceData = None


@dataclass
class TypeHierarchicalTreeNode(TcBaseObj):
    """
    Holds internal name and display name of type as well as its all immidiate sub types ordered by display name.
    
    :var typeInternalName: Internal name of type.
    :var typeDisplayName: Display name of type.
    :var subTypes: Immidiate sub types ordered by display name.
    """
    typeInternalName: str = ''
    typeDisplayName: str = ''
    subTypes: List[TypeHierarchicalTreeNode] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AuthorizationInfo(TcBaseObj):
    """
    This structure contains the accessor object, rule domain name and list of applications or utilities.
    
    :var accessorTag: Group or Role in the Group to which authorization rules need to be set.
    :var ruleDomain: Rule domain name. Valid value is either "application" or "utiltiy".
    :var names: A list of application IDs or utility IDs.
    """
    accessorTag: BusinessObject = None
    ruleDomain: str = ''
    names: List[str] = ()


@dataclass
class AccessorAccessibleNamesList(TcBaseObj):
    """
    This structure contains a list of AccessorAccessibleNamesResponse objects one for each of the given accessors and
    ServiceData.
    
    :var output: List of AccessorAccessibleNamesResponse objects one for each of the given accessors.
    :var partialErrors: The object that contains any partial errors that occur while getting the authorization rules
    for any of the given accessor.
    """
    output: List[AccessorAccessibleNamesResponse] = ()
    partialErrors: ServiceData = None


@dataclass
class AccessorAccessibleNamesResponse(TcBaseObj):
    """
    This structure contains the given accessor and rule domain and corresponding accessible applications or utilities
    (authorization rules) retrieved. If the rule domain string is "application" then the accessibleNames contains the
    application names. If the domain string is "utility" then the accessibleNames will contain the utility names.
    
    :var accessorTag: Accessor whose authorization rules need to be retrieved. This object can be either a Group or a
    Role in the group.
    :var ruleDomain: Rule domain name. Valid values for this string are "utility" and "application".
    :var accessibleNames: List of accessible application names or utility names for the given accessor.
    """
    accessorTag: BusinessObject = None
    ruleDomain: str = ''
    accessibleNames: List[str] = ()


@dataclass
class AccessorInfo(TcBaseObj):
    """
    This structure contains the accessor tag and rule domain name.
    
    :var accessorTag: Group or Role in the group to which authorization rules need to be set.
    :var ruleDomain: Rule domain name. Valid value is either "application" or "utiltiy".
    """
    accessorTag: BusinessObject = None
    ruleDomain: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Cfg0ConfiguratorPerspective
from tcsoa.gen.Internal.ProductConfiguratorAw._2018_05.ConfiguratorManagement import VariantConfigurationViewIn, VariantExpressionDataResponse, VariantConfigurationViewResponse, SelectedExpression, VariantExpressionDataInput, ValidateProductConfigResponse, FilterPanelDataResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def variantConfigurationView(cls, input: VariantConfigurationViewIn) -> VariantConfigurationViewResponse:
        """
        This operation retrieves variant configuration data required to create a custom variant rule.
        For a given Cfg0ConfiguratorPerspective and selected context or a seed variant rule, the following is retrieved:
        &bull;The valid selections for given groups by optionally applying constraints
        &bull;All scope objects (Cfg0AbsFamilyGroup, Ptn0Partition) available in the current Configurator Context
        &bull;Violations and other indicators like incomplete, invalid, package.
        &bull;The payload that should be retained by the client for subsequent calls.
        """
        return cls.execute_soa_method(
            method_name='variantConfigurationView',
            library='Internal-ProductConfiguratorAw',
            service_date='2018_05',
            service_name='ConfiguratorManagement',
            params={'input': input},
            response_cls=VariantConfigurationViewResponse,
        )

    @classmethod
    def getFilterPanelData(cls, selectedObject: BusinessObject) -> FilterPanelDataResponse:
        """
        This operation returns the filter panel data which is necessary to render filter panel on the client.
        
        Exceptions:
        >The following service exception may be returned.
        
        333003 No callback is registered to retrieve the configurator context for Business Objects of the given type.
        333008 Only objects of type Awb0Element and its subtypes are supported.
        """
        return cls.execute_soa_method(
            method_name='getFilterPanelData',
            library='Internal-ProductConfiguratorAw',
            service_date='2018_05',
            service_name='ConfiguratorManagement',
            params={'selectedObject': selectedObject},
            response_cls=FilterPanelDataResponse,
        )

    @classmethod
    def getVariantExpressionData(cls, variantExpressionDataInput: VariantExpressionDataInput) -> VariantExpressionDataResponse:
        """
        This operation returns the variant configuration expression data of the input objects.
        """
        return cls.execute_soa_method(
            method_name='getVariantExpressionData',
            library='Internal-ProductConfiguratorAw',
            service_date='2018_05',
            service_name='ConfiguratorManagement',
            params={'variantExpressionDataInput': variantExpressionDataInput},
            response_cls=VariantExpressionDataResponse,
        )

    @classmethod
    def setVariantExpressionData(cls, selectedExpressions: List[SelectedExpression], configPerspective: Cfg0ConfiguratorPerspective) -> ServiceData:
        """
        This operation sets the configuration expressions for the input business objects. The variant expressions can
        be set for variant conditions, rules and variant criteria.
        """
        return cls.execute_soa_method(
            method_name='setVariantExpressionData',
            library='Internal-ProductConfiguratorAw',
            service_date='2018_05',
            service_name='ConfiguratorManagement',
            params={'selectedExpressions': selectedExpressions, 'configPerspective': configPerspective},
            response_cls=ServiceData,
        )

    @classmethod
    def validateProductConfigurations(cls, configPerspective: Cfg0ConfiguratorPerspective, applyConstraints: int, selectedExpressions: List[SelectedExpression]) -> ValidateProductConfigResponse:
        """
        This operation validates the input selections by applying the constraint rules such as inclusion rule, 
        exclusion rule or feasibility rule for the input configurator perspective.
        The operation also returns validation status, violations and selections by considering input selections and the
        rules in the context. The response parameter 'criteriaStatus' conveys the information about the status of the
        configuration which is either valid or invalid. The parameter 'valueToViolationsMap' represents map of values
        having voilations. Each value in this map have list of vloilations. The paramter 'UserSelectionMap2' conveys
        the information about family to option values. This map contains entries accordance with the given input
        selections and the rules in the context.
        
        Exceptions:
        >The following service exception may be returned.
        
        77073     The operation has failed, because an invalid Configurator Perspective was passed.
        77074     The operation must contain a product item in the Configurator Perspective.
        256054   The value for the applyConstraints bit pattern is invalid.
        """
        return cls.execute_soa_method(
            method_name='validateProductConfigurations',
            library='Internal-ProductConfiguratorAw',
            service_date='2018_05',
            service_name='ConfiguratorManagement',
            params={'configPerspective': configPerspective, 'applyConstraints': applyConstraints, 'selectedExpressions': selectedExpressions},
            response_cls=ValidateProductConfigResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GlobalConstantValue(TcBaseObj):
    """
    The 'GlobalConstantValue' data structure holds a the name of the global constant corresponding global constant value
    
    :var key: Name of the global constant.
    :var value: The global constant value corresponding to the specified constant.
    """
    key: str = ''
    value: str = ''


@dataclass
class GlobalConstantValueResponse(TcBaseObj):
    """
    Holds the response for the getGlobalConstantValues operation.
    
    :var constantValues: The requested global constants.
    :var serviceData: This contains the status of the operation. A partial error is returned if the name global
    constant cannot be added to the global default cache (74502), or if the named constant is multivalued(74528) .
    """
    constantValues: List[GlobalConstantValue] = ()
    serviceData: ServiceData = None


@dataclass
class PropertyConstantKey(TcBaseObj):
    """
    Holds the name of the constant, name of the type and name of the property which are required to get the value of a
    property constant.
    
    :var constantName: Name of the constant.
    :var typeName: Name of the type.
    :var propertyName: Name of the property.
    """
    constantName: str = ''
    typeName: str = ''
    propertyName: str = ''


@dataclass
class PropertyConstantValue(TcBaseObj):
    """
    Holds a the name of the property constant and corresponding property constant value
    
    :var key: The requested property constant.
    :var value: The property constant value corresponding to the specified constant
    """
    key: PropertyConstantKey = None
    value: str = ''


@dataclass
class PropertyConstantValueResponse(TcBaseObj):
    """
    The 'PropertyConstantValueResponse' data structure holds the response for the 'getPropertyConstantValues' method.
    
    :var constantValues: The resultant property constant values are returned as keyvalue pairs using
    'PropertyConstantValue' structure.
    :var serviceData: This contains the status of the operation.
    """
    constantValues: List[PropertyConstantValue] = ()
    serviceData: ServiceData = None


@dataclass
class TypeConstantKey(TcBaseObj):
    """
    Holds the name of the constant, name of the type which are required to get the value of a type constant.
    
    :var constantName: Name of the constant.
    :var typeName: Name of the type.
    """
    constantName: str = ''
    typeName: str = ''


@dataclass
class TypeConstantValue(TcBaseObj):
    """
    Holds a the name of the type constant and corresponding type constant value.
    
    :var key: The requested type constant.
    :var value: The type constant value corresponding to the specified constant.
    """
    key: TypeConstantKey = None
    value: str = ''


@dataclass
class TypeConstantValueResponse(TcBaseObj):
    """
    Holds the response for the 'getTypeConstantValues' operation.
    
    :var constantValues: The resultant type constant values are returned as key/value pairs using 'TypeConstantValue'
    structure.
    :var serviceData: This contains the status of the operation.
    """
    constantValues: List[TypeConstantValue] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportConfiguredNXAssemblyInfo(TcBaseObj):
    """
    Contains the BOMLine object reference and naming format.
    
    :var bomline: Object reference of the BOMLine.
    :var namingFormat: The naming format can be defined by the user in the User Interface.  The option is to Add a
    Prefix or to Auto translate.
    """
    bomline: BOMLine = None
    namingFormat: str = ''


@dataclass
class ExportConfiguredNXAssemblyOutput(TcBaseObj):
    """
    Contains zipfileticket, logfileticket, and a list of 'ExportConfiguredNXAssemblyInfo' structures.  The
    'ExportConfiguredNXAssemblyInfo' structure contains the input BOMLine object and the naming format
    
    :var transientZipFileReadTicket: Exported zip file ticket.
    :var transientLogFileReadTicket: Exported log file ticket.
    :var bomline: Input BOMLine.
    """
    transientZipFileReadTicket: str = ''
    transientLogFileReadTicket: str = ''
    bomline: BOMLine = None


@dataclass
class ExportConfiguredNXAssemblyResponse(TcBaseObj):
    """
    Consists of the output and the 'serviceData'
    The output will be returned as a list of 'ExportConfiguredNXAssemblyOutput' objects each of which contains
    zipfileticket, logfileticket, and a list of 'ExportConfiguredNXAssemblyInfo' structures.  The
    'ExportConfiguredNXAssemblyInfo' structure contains the input BOMLine object and the naming format.
    
    
    :var output: A list of type 'ExportConfiguredNXAssemblyOutput'
    :var serviceData: The input BOMLine and the error message.
    """
    output: List[ExportConfiguredNXAssemblyOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.ProjectManagement._2014_06.ScheduleManagement import FilteredUsersInfo, FilterCriteria
from tcsoa.utils import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def filterUsers(cls, filteringcriteria: FilterCriteria) -> FilteredUsersInfo:
        """
        In Schedule Manager, Discipline, Group, Role and Fnd0Qualification can be assigned to the ScheduleTask as
        placeholder assignments. These placeholder assignments can then be assigned to specific users who are a part of
        the placeholder assignment. 
        
        The operation returns a list of Users which match the input filter criteria of Discipline, Group, Role and
        Fnd0Qualification. 
        """
        return cls.execute_soa_method(
            method_name='filterUsers',
            library='ProjectManagement',
            service_date='2014_06',
            service_name='ScheduleManagement',
            params={'filteringcriteria': filteringcriteria},
            response_cls=FilteredUsersInfo,
        )



from __future__ import annotations

from tcsoa.gen.StructureManagement._2008_03.Composition import AssignChildLinesResponse, AssignChildLinesParameter
from typing import List
from tcsoa.utils import TcService


class CompositionService(TcService):

    @classmethod
    def assignChildLines(cls, input: List[AssignChildLinesParameter]) -> AssignChildLinesResponse:
        """
        Assign lines from one or more parent to a new parent line.
        """
        return cls.execute_soa_method(
            method_name='assignChildLines',
            library='StructureManagement',
            service_date='2008_03',
            service_name='Composition',
            params={'input': input},
            response_cls=AssignChildLinesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset, Item, BusinessObject, TableDefinition, ItemRevision
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetItemFromAttributeInfo(TcBaseObj):
    """
    Input parameters for retrieving Item and ItemRevision objects based on attribute keys
    
    :var itemAttributes: A map of attribute names and values ('string/string') used to peform the search. Typical keys
    are "item_id", "object_type", "object_name", "owning_organization", and "date_released".  A special key, "rev_id",
    may also be used to specify a single related ItemRevision object to retrieve. This "rev_id" key is effective only
    if the nRev parameter equals zero.
    :var revIds: A list of revision ID strings specifying ItemRevison objects to retrieve. This parameter only takes
    effect if the 'nRev' parameter equals zero. This list of revision IDs can be used independent of the "rev_id" key
    value in 'itemAttributes'
    """
    itemAttributes: GetItemAttributeMap = None
    revIds: List[str] = ()


@dataclass
class GetItemFromAttributeItemOutput(TcBaseObj):
    """
    This data structure contains an Item and a list of related ItemRevision objects retrieved by a
    'getItemFromAttribute' operation.
    
    :var item: The retrieved Item object
    :var itemRevOutput: The list of related ItemRevision objects
    """
    item: Item = None
    itemRevOutput: List[GetItemFromAttributeItemRevOutput] = ()


@dataclass
class GetItemFromAttributeItemRevOutput(TcBaseObj):
    """
    Contains a single ItemRevision retrieved by the getItemFromAttribute operation
    
    :var itemRevision: The retrieved ItemRevision
    :var datasets: list of Datasets associated with the ItemRevision
    """
    itemRevision: ItemRevision = None
    datasets: List[Dataset] = ()


@dataclass
class GetItemFromAttributeResponse(TcBaseObj):
    """
    The return structure from from 'getItemFromAttribute' operation. Contains a list of
    'GetItemFromAttributeItemOutput' structures.
    
    :var output: A list of found Item and ItemRevision objects
    :var serviceData: Standard 'ServerData' member with any returned error codes and messages
    """
    output: List[GetItemFromAttributeItemOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetTablePropertiesResponse(TcBaseObj):
    """
    Structure holds response from getTableProperties.
    
    :var tableInfo: This vector contains a list of TableInfo, and each TableInfo contains information pertaining to the
    specific Table it references.
    :var serviceData: The ServiceData structure is used to return the updated Table objects in the update section and
    also any errors encountered during this operation call.
    """
    tableInfo: List[TableInfo] = ()
    serviceData: ServiceData = None


@dataclass
class TableCellInfo(TcBaseObj):
    """
    This structure contains Table Cell Info.
    
    :var row: This specifies the row number of the cell in the Table.
    :var column: This specifies the column number of the cell in the Table.
    :var desc: This is a string which can be used to capture a description for the cell.
    :var value: This is a structure which contains information pertaining to a value on the cell.
    """
    row: int = 0
    column: int = 0
    desc: str = ''
    value: TableCellValueTypeInfo = None


@dataclass
class TableCellValueTypeInfo(TcBaseObj):
    """
    This structure contains table cell value type info.
    
    :var type: This specifies the data type that the cell of the table can hold. The type specified in this string,
    should be one of the below supported type:
    - TableCellInt
    - TableCellString
    - TableCellDouble
    - TableCellLogical
    - TableCellHex
    - TableCellSED
    - TableCellBCD
    - TableCellDate
    
    
    :var strValues: A list of values
    """
    type: str = ''
    strValues: List[str] = ()


@dataclass
class TableDefInfo(TcBaseObj):
    """
    This structure contains Table Definition Info.
    
    :var rows: This specifies the number of rows in the Table.
    :var columns: This specifies the number of columns in the Table.
    :var rowLabels: This is a vector of strings, each string representing the labels corresponding to a row in the
    Table.
    :var colLabels: This is a vector of strings, each string representing the labels corresponding to a column in the
    Table.
    :var tableDef: tableDef is of type TableDefinition
    """
    rows: int = 0
    columns: int = 0
    rowLabels: List[str] = ()
    colLabels: List[str] = ()
    tableDef: TableDefinition = None


@dataclass
class TableInfo(TcBaseObj):
    """
    Table Info
    
    :var tableObject: The Business Object corresponding to the Table
    :var tableDefInfo: Meta information about the Table such as the row labels, columns labels, and its size are
    contained in this structure.
    :var tableCells: This is a vector of the structures that contain information pertaining to each of the cells in the
    Table.
    """
    tableObject: BusinessObject = None
    tableDefInfo: TableDefInfo = None
    tableCells: List[TableCellInfo] = ()


"""
A hash map containing the key-value pairs specifying the Item objects to search for. Used by the 'GetItemFromAttributeInfo' structure to store the Item search criteria.
"""
GetItemAttributeMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Cad._2007_12.DataManagement import CreateOrUpdatePartsPref
from tcsoa.gen.Cad._2008_06.DataManagement import CreateOrUpdatePartsResponse
from typing import List
from tcsoa.gen.Cad._2010_09.DataManagement import PartInfo
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def createOrUpdateParts(cls, partInput: List[PartInfo], pref: CreateOrUpdatePartsPref) -> CreateOrUpdatePartsResponse:
        """
        CreateOrUpdateParts allows the user to update or create a set of parts using Items, Item Revisions, and
        Datasets and save the boudingbox information related to these objects.
        The service first attempts to validate the existence of the Item, Item Revision, and Dataset.
        If the Item and Item Revision already exist, but the Dataset does not, then only the Dataset is created.
        If the Dataset exists, a new version will be added to the existing series.
        If any of the objects exist but the input attribute values that differ from those already set, attempts are
        made to update the values if possible.
        If the boundingbox information exists it saves that information on that particular dataset else it willnot save
        the boudingbox information.
        If no item object reference or item revision object references are specified then a new item and item revision
        and related objects will be created.
        All objects created and updated will be returned in the ServiceData.
        All partial errors will contain the clientIDs for all items related to the error message, i.e. if a dataset
        encounters an error, then the ID for that erro will be the item client ID concantentated with the item revision
        id contantenated with the dataset client ID, all separated by semi-colons ( ; ).
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateParts',
            library='Cad',
            service_date='2010_09',
            service_name='DataManagement',
            params={'partInput': partInput, 'pref': pref},
            response_cls=CreateOrUpdatePartsResponse,
        )



from tcsoa.gen.Internal_IcsAw._2017_06.services import AuthorService as imp0
from tcsoa.gen.Internal_IcsAw._2018_05.services import ClassificationService as imp1
from tcsoa.gen.Internal_IcsAw._2018_12.services import ClassificationService as imp2
from tcsoa.gen.Internal_IcsAw._2019_12.services import ClassificationService as imp3
from tcsoa.gen.Internal_IcsAw._2017_12.services import AuthorService as imp4
from tcsoa.gen.Internal_IcsAw._2019_06.services import ClassificationService as imp5
from tcsoa.utils import TcService


class AuthorService(TcService):
    createOrUpdateClassificationObjects = imp0.createOrUpdateClassificationObjects
    findClassifications = imp4.findClassifications


class ClassificationService(TcService):
    deleteClassificationObjects = imp1.deleteClassificationObjects
    findClassificationInfo = imp1.findClassificationInfo
    findClassificationInfo2 = imp2.findClassificationInfo2
    findClassificationInfo3 = imp3.findClassificationInfo3
    getClassificationCmdVisibilityInfo = imp5.getClassificationCmdVisibilityInfo
    saveClassificationObjects = imp1.saveClassificationObjects
    saveClassificationObjects2 = imp2.saveClassificationObjects2



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, BOMLine
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class MatchLineInputData(TcBaseObj):
    """
    The MatchLineInputData structure represents all of the data necessary to search any ItemRevision object from all
    open BOMWindow instances.
    
    :var revObject: The business revision object for which matching BOMLine needs to be searched.
    :var scopeTopLines: The list of top BOMLine of all open BOMWindow instance(s) in which revision objects needs to be
    searched.
    """
    revObject: BusinessObject = None
    scopeTopLines: List[BusinessObject] = ()


@dataclass
class MatchedLineData(TcBaseObj):
    """
    This structure contains information about matching BOMLine or GDELine, and top line within the scope of BOMWindow.
    
    :var matchedLine: First matched BOMLine or GDELine object. If no matching object found then returns null.
    :var topLine: Top line of BOMWindow in which the matching BOMLine found.
    """
    matchedLine: BusinessObject = None
    topLine: BusinessObject = None


@dataclass
class MatchedLineOutput(TcBaseObj):
    """
    This structure represents the output details for each matching BOMLine or GDELine. It has information about
    underlying revision object, list of matching BOMLine instances, and its corresponding top BOMLine.
    
    :var revObject: The revision object which searched in all given BOMWindow instance(s).
    :var matchedLineData: This is the list of found matching BOMLine and their corresponding BOMWindow top line for
    given revision.
    """
    revObject: BusinessObject = None
    matchedLineData: List[MatchedLineData] = ()


@dataclass
class MatchedLineResponse(TcBaseObj):
    """
    This structure represents the output of getMatchingLines operation.  It has information about matched BOMLine
    objects and its corresponding revision object.
    
    :var matchedOutput: A list of MatchedLineOutput structure containing information of each matching found details.
    :var serviceData: The Service Data
    """
    matchedOutput: List[MatchedLineOutput] = ()
    serviceData: ServiceData = None


@dataclass
class TraceabilityMatrixInfo1(TcBaseObj):
    """
    This structure holds information to create the traceability matrix between source and target BOM structure.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var sourceLine: The source BOMLine object, required to create a traceability matrix.
    :var targetLine: The target BOMLine object, required to create a traceability matrix.
    :var filterFormat: Name of filtering format for traceability matrix. On this format name will be decided whether to
    filter or not and also need to include subtypes of trace link for selected trace link types. This filterFormat
    strings will be:
    NO_FILTER : Filter will not be applied for getting trace link relations. It will get all trace link relations.
    DO_FILTER : Filter will be applied for getting only selected trace link types mentioned in parameter filterTypes
    list.
    DO_FILTER_SUBTYPE : Filter will be applied for getting only selected trace link type objects mentioned in list
    filterTypes, including their subtypes.
    :var filterTypes: Vector of trace link types to be filtered and displayed in traceability matrix.
    """
    clientId: str = ''
    sourceLine: BOMLine = None
    targetLine: BOMLine = None
    filterFormat: str = ''
    filterTypes: List[str] = ()



from __future__ import annotations

from tcsoa.utils import TcService


class PreferenceManagementService(TcService):

    @classmethod
    def refreshPreferences(cls) -> bool:
        """
        Refreshes the preference values stored in the server cache, so that they are synchronized with the latest
        values. 
        
        This situation might happen when the preferences for a given user are being changed in 2 different sessions, or
        when an administrator is making changes to the Site / Role or Group preferences. 
        <Calling the refreshPreferences operation will retrieve the updated values.
        
        Exceptions:
        >If the operation has failed.
        """
        return cls.execute_soa_method(
            method_name='refreshPreferences',
            library='Administration',
            service_date='2011_05',
            service_name='PreferenceManagement',
            params={},
            response_cls=bool,
        )



from __future__ import annotations

from tcsoa.gen.Administration._2010_04.IRM import ExtraProtectionInfoResponse, PrivNamesInfoResponse, AccessorTypesResponse, ACLInfoResponse
from tcsoa.gen.BusinessObjects import BusinessObject, User
from tcsoa.gen.Administration._2010_04.DisciplineManagement import GetDisciplineResponse
from typing import List
from tcsoa.utils import TcService


class IRMService(TcService):

    @classmethod
    def getAccessorTypes(cls) -> AccessorTypesResponse:
        """
        This operation can be used to get the names of all the Accessor Types in Teamcenter. Accessor Types are used to
        configure access privileges for different accessors in Access Control List (ACL) table in Access Manager
        Application. Examples for the Accessor Types are "World", "User", "Group", and "Role in Group".  For more
        information on the Accessor Types please refer to the Access Manager guide in Teamcenter documentation. The
        returned names from this operation will include both internal names and corresponding client locale specific
        localized display names of the Accessor Types.  The display names of the Accessor Types are used for displaying
        in the User Interface.  Whereas, the internal names of the Accessor Types are used for internal processing of
        the rule tree evaluation.
        
        Use cases:
        In general wherever the Accessor Type names need to be displayed this operation can be used. At present
        following use cases in Teamcenter Rich Application Client (RAC) calls this operation.
        
        - Display the Accessor Type names in Extra Protection information dialog.
        - Display the Accessor Type names in Effective ACL dialog.
        
        """
        return cls.execute_soa_method(
            method_name='getAccessorTypes',
            library='Administration',
            service_date='2010_04',
            service_name='IRM',
            params={},
            response_cls=AccessorTypesResponse,
        )

    @classmethod
    def getEffectiveACLInfo2(cls, objects: List[BusinessObject]) -> ACLInfoResponse:
        """
        This operation can be used to get the effective Access Control List (ACL) information for a list of business
        objects.  Effective ACL information displays the Access Control Entries (ACEs) that are applicable to the
        business object with respect to the user for whom access privileges are being evaluated on the object.
        Applicable ACEs are picked up from the ACLs that are configured against the Access Manager Rules which are
        evaluated to true for the given business object based on details like the object's type, object's class, object
        attributes, object status and project to which it is assigned.  By looking at the effective ACL table end user
        will be able to understand what privileges or granted, what privileges are denied for the user on the object
        and, what ACL and Accessor Type are involved in either granting or denying a particular access privilege. For
        more information on ACLs, ACEs, effective ACLs please refer to the Access Manager guide in Teamcenter.
        
        Use cases:
        ACL list dialog in Teamcenter Rich Application Client (RAC) calls this operation to get the effective ACL
        information.
        """
        return cls.execute_soa_method(
            method_name='getEffectiveACLInfo2',
            library='Administration',
            service_date='2010_04',
            service_name='IRM',
            params={'objects': objects},
            response_cls=ACLInfoResponse,
        )

    @classmethod
    def getExtraProtectionInfo2(cls, user: User, objects: List[BusinessObject]) -> ExtraProtectionInfoResponse:
        """
        This operation can be used to get the additional access protection information for a given user on a set of
        business objects.  Additional protection information can be used to understand what Named ACL, what Accessor
        and what AM rule path are involved in arriving at the verdict for an access privilege on a given object for the
        given user. This helps the user to understand why a particular privilege on the given object is granted or
        denied for the given user.  At present this information is displayed only in RAC client in the "Extra
        Protection Information" dialog.  This operation supports localization of privilege names, ACL names and
        Accessor type names.
        
        Use cases:
        "Extra Protection Information" dialog in Teamcenter Rich Application Client (RAC)  calls this operation.
        """
        return cls.execute_soa_method(
            method_name='getExtraProtectionInfo2',
            library='Administration',
            service_date='2010_04',
            service_name='IRM',
            params={'user': user, 'objects': objects},
            response_cls=ExtraProtectionInfoResponse,
        )

    @classmethod
    def getPrivilegeNames(cls) -> PrivNamesInfoResponse:
        """
        This operation can be used to get the internal names and corresponding display values of all the access
        privileges in Teamcenter.  The display names of the privileges are used to display the privilege names in the
        User Interface in client specific locale.  Whereas, the internal privilege names are used for internal
        processing of the rule tree evaluation. Below is the list of access privileges in Teamcenter release 10.0. For
        functional information on each of these privileges please refer to the Access Manager guide in Teamcenter
        documentation.
        
        ADD_CONTENT
        ASSIGN_TO_PROJECT
        Administer_ADA_Licenses
        BATCH_PRINT
        CHANGE
        CHANGE_OWNER
        CICO
        COPY
        DELETE
        DEMOTE
        DIGITAL_SIGN
        EXPORT
        IMPORT
        IP_ADMIN
        IP_Classifier
        ITAR_ADMIN
        ITAR_Classifier
        MARKUP
        PROMOTE
        PUBLISH
        READ
        REMOTE_CICO
        REMOVE_CONTENT
        REMOVE_FROM_PROJECT
        SUBSCRIBE
        TRANSFER_IN
        TRANSFER_OUT
        TRANSLATION
        UNMANAGE
        WRITE
        WRITE_ICOS
        
        Use cases:
        This operation can be used in general wherever the privilege names need to be displayed. At present following
        use cases in Teamcenter Rich Application Client (RAC) calls this operation.
        
        - Display the privilege names in Access dialog.
        - Display the privilege names in Extra Protection information dialog.
        - Display the privilege names in effective Access Control List (ACL) dialog.
        - Display the privilege tool tips in Named ACL panel.
        
        """
        return cls.execute_soa_method(
            method_name='getPrivilegeNames',
            library='Administration',
            service_date='2010_04',
            service_name='IRM',
            params={},
            response_cls=PrivNamesInfoResponse,
        )


class DisciplineManagementService(TcService):

    @classmethod
    def getDiscipline(cls, disciplineName: str) -> GetDisciplineResponse:
        """
        This operation gets the Discipline object with given name. If no discipline object is found with the given
        name, the returned discipline object would be null.
        """
        return cls.execute_soa_method(
            method_name='getDiscipline',
            library='Administration',
            service_date='2010_04',
            service_name='DisciplineManagement',
            params={'disciplineName': disciplineName},
            response_cls=GetDisciplineResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ObjectCoverageInput(TcBaseObj):
    """
    Structure contains input parameters for verifyObjectCoverageByRule operation
    
    :var contexts: scoped lines in the target window
    :var objectsToCheck: the list of lines,that will be checked against the supplied closure rule
    :var closureRule: the closure rule selected by the user
    :var depth: the traversal depth entered by the user
    :var otherStructure: context of another structure (can be window or any line in it)
    """
    contexts: List[BusinessObject] = ()
    objectsToCheck: List[BusinessObject] = ()
    closureRule: str = ''
    depth: int = 0
    otherStructure: BusinessObject = None


@dataclass
class ObjectCoverageResponse(TcBaseObj):
    """
    Structure contains output parameters for verifyObjectCoverageByRule operation
    
    :var covered: return vector (corresponds to the input objects vector) - with true or false
    :var serviceData: contains any errors received during the execution
    """
    covered: List[bool] = ()
    serviceData: ServiceData = None


@dataclass
class TraversedObjectsInput(TcBaseObj):
    """
    structure contains input parameters for getTraversedObjectsByRule operation
    
    :var scopeObjects: the scope lines selected by the user
    :var closureRule: the closure rule selected by the user
    :var depth: the traversal depth entered by the user
    :var otherStructure: target context (can be window or any line in it)
    """
    scopeObjects: List[BusinessObject] = ()
    closureRule: str = ''
    depth: int = 0
    otherStructure: BusinessObject = None


@dataclass
class TraversedObjectsResponse(TcBaseObj):
    """
    structure contains output parameters for getTraversedObjectsByRule operation
    
    :var resultObjects: return vector of the the auto-expanded/filtered lines
    :var serviceData: return errors if verifacation was failed or illegal
    """
    resultObjects: List[BusinessObject] = ()
    serviceData: ServiceData = None



from typing import Dict


class TcSoaConfig:
    backend = None
    date_format = r'%d.%m.%Y %H:%M:%S'
    global_obj_cache_enabled = True

    __gobj_cache = dict()

    @classmethod
    def internal_handle_sd(cls, sd):
        """ Handles incoming service data, and updates the cache """
        if hasattr(sd, 'modelObjects') and sd.modelObjects:
            for uid, obj in sd.modelObjects.items():
                if uid not in cls.__gobj_cache:
                    cls.__gobj_cache[uid] = dict(uid=uid, type=obj['type'])
                for pname, pval in obj.props.items():
                    cls.__gobj_cache[uid][pname] = pval

    @classmethod
    def internal_get_prop(cls, uid: str, prop_name: str):
        props = cls.__gobj_cache.get(uid, None)
        if props:
            return props.get(prop_name, None)
        return None

    @classmethod
    def internal_get_obj(cls, uid: str) -> Dict[str, any]:
        return cls.__gobj_cache.get(uid, None)



from __future__ import annotations

from typing import List
from tcsoa.gen.BusinessObjects import RuntimeBusinessObject
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class HasActiveMarkupAssociatedOut(TcBaseObj):
    """
    Output of the hasActiveMarkupAssociated operation.
    
    :var hasActiveMarkup: A Boolean variable which contains:
    -  true: if there are active markups.
    -  false: if there are no active markups.
    
    
    :var serviceData: An object of ServiceData which contains any errors that may have occurred during operation.
    """
    hasActiveMarkup: bool = False
    serviceData: ServiceData = None


@dataclass
class MassUpdateChange(TcBaseObj):
    """
    Holds the name and new value of the property to be modified.
    
    :var propName: Name of the property which is to be modified.
    :var propValue: New value for the property specified in propName.
    """
    propName: str = ''
    propValue: str = ''


@dataclass
class SaveImpactedAssembliesIn(TcBaseObj):
    """
    Input structure for the saveImpactedAssemblies operation.
    
    :var impactedObject: Fnd0MUImpactedParents Object to specify modified impacted assembly.
    :var massUpdateChanges: List of MassUpdateChange objects.
    """
    impactedObject: RuntimeBusinessObject = None
    massUpdateChanges: List[MassUpdateChange] = ()
